info = {
    "name": "bm",
    "date_order": "DMY",
    "january": [
        "zan",
        "zanwuye"
    ],
    "february": [
        "feb",
        "feburuye"
    ],
    "march": [
        "mar",
        "marisi"
    ],
    "april": [
        "awi",
        "awirili"
    ],
    "may": [
        "mɛ"
    ],
    "june": [
        "zuw",
        "zuwɛn"
    ],
    "july": [
        "zul",
        "zuluye"
    ],
    "august": [
        "uti"
    ],
    "september": [
        "sɛt",
        "sɛtanburu"
    ],
    "october": [
        "ɔku",
        "ɔkutɔburu"
    ],
    "november": [
        "now",
        "nowanburu"
    ],
    "december": [
        "des",
        "desanburu"
    ],
    "monday": [
        "ntɛ",
        "ntɛnɛ"
    ],
    "tuesday": [
        "tar",
        "tarata"
    ],
    "wednesday": [
        "ara",
        "araba"
    ],
    "thursday": [
        "ala",
        "alamisa"
    ],
    "friday": [
        "jum",
        "juma"
    ],
    "saturday": [
        "sib",
        "sibiri"
    ],
    "sunday": [
        "kar",
        "kari"
    ],
    "am": [
        "am"
    ],
    "pm": [
        "pm"
    ],
    "year": [
        "san"
    ],
    "month": [
        "kalo"
    ],
    "week": [
        "dɔgɔkun"
    ],
    "day": [
        "don"
    ],
    "hour": [
        "lɛrɛ"
    ],
    "minute": [
        "miniti"
    ],
    "second": [
        "sekondi"
    ],
    "relative-type": {
        "0 day ago": [
            "bi"
        ],
        "0 hour ago": [
            "this hour"
        ],
        "0 minute ago": [
            "this minute"
        ],
        "0 month ago": [
            "this month"
        ],
        "0 second ago": [
            "now"
        ],
        "0 week ago": [
            "this week"
        ],
        "0 year ago": [
            "this year"
        ],
        "1 day ago": [
            "kunu"
        ],
        "1 month ago": [
            "last month"
        ],
        "1 week ago": [
            "last week"
        ],
        "1 year ago": [
            "last year"
        ],
        "in 1 day": [
            "sini"
        ],
        "in 1 month": [
            "next month"
        ],
        "in 1 week": [
            "next week"
        ],
        "in 1 year": [
            "next year"
        ]
    },
    "locale_specific": {},
    "skip": [
        " ",
        "'",
        ",",
        "-",
        ".",
        "/",
        ";",
        "@",
        "[",
        "]",
        "|",
        "，"
    ]
}



info = {
    "name": "bs-Cyrl",
    "date_order": "DMY",
    "january": [
        "јан",
        "јануар"
    ],
    "february": [
        "феб",
        "фебруар"
    ],
    "march": [
        "мар",
        "март"
    ],
    "april": [
        "апр",
        "април"
    ],
    "may": [
        "мај"
    ],
    "june": [
        "јун",
        "јуни"
    ],
    "july": [
        "јул",
        "јули"
    ],
    "august": [
        "авг",
        "август"
    ],
    "september": [
        "сеп",
        "септембар"
    ],
    "october": [
        "окт",
        "октобар"
    ],
    "november": [
        "нов",
        "новембар"
    ],
    "december": [
        "дец",
        "децембар"
    ],
    "monday": [
        "пон",
        "понедељак"
    ],
    "tuesday": [
        "уто",
        "уторак"
    ],
    "wednesday": [
        "сри",
        "сриједа"
    ],
    "thursday": [
        "чет",
        "четвртак"
    ],
    "friday": [
        "пет",
        "петак"
    ],
    "saturday": [
        "суб",
        "субота"
    ],
    "sunday": [
        "нед",
        "недеља"
    ],
    "am": [
        "пре подне"
    ],
    "pm": [
        "поподне"
    ],
    "year": [
        "година"
    ],
    "month": [
        "месец"
    ],
    "week": [
        "недеља"
    ],
    "day": [
        "дан"
    ],
    "hour": [
        "час"
    ],
    "minute": [
        "минут"
    ],
    "second": [
        "секунд"
    ],
    "relative-type": {
        "0 day ago": [
            "данас"
        ],
        "0 hour ago": [
            "this hour"
        ],
        "0 minute ago": [
            "this minute"
        ],
        "0 month ago": [
            "овог месеца"
        ],
        "0 second ago": [
            "now"
        ],
        "0 week ago": [
            "ове недеље"
        ],
        "0 year ago": [
            "ове године"
        ],
        "1 day ago": [
            "јуче"
        ],
        "1 month ago": [
            "прошлог месеца"
        ],
        "1 week ago": [
            "прошле недеље"
        ],
        "1 year ago": [
            "прошле године"
        ],
        "in 1 day": [
            "сутра"
        ],
        "in 1 month": [
            "следећег месеца"
        ],
        "in 1 week": [
            "следеће недеље"
        ],
        "in 1 year": [
            "следеће године"
        ]
    },
    "relative-type-regex": {
        "\\1 day ago": [
            "пре (\\d+[.,]?\\d*) дан",
            "пре (\\d+[.,]?\\d*) дана"
        ],
        "\\1 hour ago": [
            "пре (\\d+[.,]?\\d*) сат",
            "пре (\\d+[.,]?\\d*) сати"
        ],
        "\\1 minute ago": [
            "пре (\\d+[.,]?\\d*) минут",
            "пре (\\d+[.,]?\\d*) минута"
        ],
        "\\1 month ago": [
            "пре (\\d+[.,]?\\d*) месец",
            "пре (\\d+[.,]?\\d*) месеци"
        ],
        "\\1 second ago": [
            "пре (\\d+[.,]?\\d*) секунд",
            "пре (\\d+[.,]?\\d*) секунди"
        ],
        "\\1 week ago": [
            "пре (\\d+[.,]?\\d*) недеља",
            "пре (\\d+[.,]?\\d*) недељу"
        ],
        "\\1 year ago": [
            "пре (\\d+[.,]?\\d*) година",
            "пре (\\d+[.,]?\\d*) годину"
        ],
        "in \\1 day": [
            "за (\\d+[.,]?\\d*) дан",
            "за (\\d+[.,]?\\d*) дана"
        ],
        "in \\1 hour": [
            "за (\\d+[.,]?\\d*) сат",
            "за (\\d+[.,]?\\d*) сати"
        ],
        "in \\1 minute": [
            "за (\\d+[.,]?\\d*) минут",
            "за (\\d+[.,]?\\d*) минута"
        ],
        "in \\1 month": [
            "за (\\d+[.,]?\\d*) месец",
            "за (\\d+[.,]?\\d*) месеци"
        ],
        "in \\1 second": [
            "за (\\d+[.,]?\\d*) секунд",
            "за (\\d+[.,]?\\d*) секунди"
        ],
        "in \\1 week": [
            "за (\\d+[.,]?\\d*) недеља",
            "за (\\d+[.,]?\\d*) недељу"
        ],
        "in \\1 year": [
            "за (\\d+[.,]?\\d*) година",
            "за (\\d+[.,]?\\d*) годину"
        ]
    },
    "locale_specific": {},
    "skip": [
        " ",
        "'",
        ",",
        "-",
        ".",
        "/",
        ";",
        "@",
        "[",
        "]",
        "|",
        "，"
    ]
}



from pathlib import Path
import urllib.request
import os
import logging

from .exceptions import FastTextModelNotFoundException
from .utils import dateparser_model_home, create_data_model_home


def fasttext_downloader(model_name):
    create_data_model_home()
    models = {
        "small": "https://dl.fbaipublicfiles.com/fasttext/supervised-models/lid.176.ftz",
        "large": "https://dl.fbaipublicfiles.com/fasttext/supervised-models/lid.176.bin"
    }
    if model_name not in models:
        message = "dateparser-download: Couldn't find a model called \"{}\". Supported models are: {}".format(
            model_name, ", ".join(models.keys())
        )
        raise FastTextModelNotFoundException(message)

    models_directory_path = os.path.join(dateparser_model_home, (model_name + ".bin"))

    if not Path(models_directory_path).is_file():
        model_url = models[model_name]
        logging.info("dateparser-download: Downloading model \"{}\" from \"{}\"...".format(model_name, model_url))
        try:
            urllib.request.urlretrieve(model_url, models_directory_path)
        except urllib.error.HTTPError as e:
            raise Exception("dateparser-download: Fasttext model cannot be downloaded due to HTTP error") from e
    else:
        logging.info("dateparser-download: The model \"{}\" is already downloaded".format(model_name))



info = {
    "name": "mgh",
    "date_order": "DMY",
    "january": [
        "kwa",
        "mweri wo kwanza"
    ],
    "february": [
        "mweri wo unayeli",
        "una"
    ],
    "march": [
        "mweri wo uneraru",
        "rar"
    ],
    "april": [
        "che",
        "mweri wo unecheshe"
    ],
    "may": [
        "mweri wo unethanu",
        "tha"
    ],
    "june": [
        "moc",
        "mweri wo thanu na mocha"
    ],
    "july": [
        "mweri wo saba",
        "sab"
    ],
    "august": [
        "mweri wo nane",
        "nan"
    ],
    "september": [
        "mweri wo tisa",
        "tis"
    ],
    "october": [
        "kum",
        "mweri wo kumi"
    ],
    "november": [
        "moj",
        "mweri wo kumi na moja"
    ],
    "december": [
        "mweri wo kumi na yel'li",
        "yel"
    ],
    "monday": [
        "jtt",
        "jumatatu"
    ],
    "tuesday": [
        "jnn",
        "jumanne"
    ],
    "wednesday": [
        "jtn",
        "jumatano"
    ],
    "thursday": [
        "ara",
        "arahamisi"
    ],
    "friday": [
        "iju",
        "ijumaa"
    ],
    "saturday": [
        "jmo",
        "jumamosi"
    ],
    "sunday": [
        "sab",
        "sabato"
    ],
    "am": [
        "wichishu"
    ],
    "pm": [
        "mchochil'l"
    ],
    "year": [
        "yaka"
    ],
    "month": [
        "mweri"
    ],
    "week": [
        "iwiki mocha"
    ],
    "day": [
        "nihuku"
    ],
    "hour": [
        "isaa"
    ],
    "minute": [
        "idakika"
    ],
    "second": [
        "isekunde"
    ],
    "relative-type": {
        "0 day ago": [
            "lel'lo"
        ],
        "0 hour ago": [
            "this hour"
        ],
        "0 minute ago": [
            "this minute"
        ],
        "0 month ago": [
            "this month"
        ],
        "0 second ago": [
            "now"
        ],
        "0 week ago": [
            "this week"
        ],
        "0 year ago": [
            "this year"
        ],
        "1 day ago": [
            "n'chana"
        ],
        "1 month ago": [
            "last month"
        ],
        "1 week ago": [
            "last week"
        ],
        "1 year ago": [
            "last year"
        ],
        "in 1 day": [
            "me'llo"
        ],
        "in 1 month": [
            "next month"
        ],
        "in 1 week": [
            "next week"
        ],
        "in 1 year": [
            "next year"
        ]
    },
    "locale_specific": {},
    "skip": [
        " ",
        "'",
        ",",
        "-",
        ".",
        "/",
        ";",
        "@",
        "[",
        "]",
        "|",
        "，"
    ]
}



info = {
    "name": "mer",
    "date_order": "DMY",
    "january": [
        "jan",
        "januarĩ"
    ],
    "february": [
        "feb",
        "feburuarĩ"
    ],
    "march": [
        "mac",
        "machi"
    ],
    "april": [
        "ĩpu",
        "ĩpurũ"
    ],
    "may": [
        "mĩĩ"
    ],
    "june": [
        "nju",
        "njuni"
    ],
    "july": [
        "njr",
        "njuraĩ"
    ],
    "august": [
        "aga",
        "agasti"
    ],
    "september": [
        "septemba",
        "spt"
    ],
    "october": [
        "okt",
        "oktũba"
    ],
    "november": [
        "nov",
        "novemba"
    ],
    "december": [
        "dec",
        "dicemba"
    ],
    "monday": [
        "mra",
        "muramuko"
    ],
    "tuesday": [
        "wai",
        "wairi"
    ],
    "wednesday": [
        "wet",
        "wethatu"
    ],
    "thursday": [
        "wen",
        "wena"
    ],
    "friday": [
        "wetano",
        "wtn"
    ],
    "saturday": [
        "jum",
        "jumamosi"
    ],
    "sunday": [
        "kiu",
        "kiumia"
    ],
    "am": [
        "rũ"
    ],
    "pm": [
        "ũg"
    ],
    "year": [
        "mwaka"
    ],
    "month": [
        "mweri"
    ],
    "week": [
        "kiumia"
    ],
    "day": [
        "ntukũ"
    ],
    "hour": [
        "ĩthaa"
    ],
    "minute": [
        "ndagika"
    ],
    "second": [
        "sekondi"
    ],
    "relative-type": {
        "0 day ago": [
            "narua"
        ],
        "0 hour ago": [
            "this hour"
        ],
        "0 minute ago": [
            "this minute"
        ],
        "0 month ago": [
            "this month"
        ],
        "0 second ago": [
            "now"
        ],
        "0 week ago": [
            "this week"
        ],
        "0 year ago": [
            "this year"
        ],
        "1 day ago": [
            "ĩgoro"
        ],
        "1 month ago": [
            "last month"
        ],
        "1 week ago": [
            "last week"
        ],
        "1 year ago": [
            "last year"
        ],
        "in 1 day": [
            "rũjũ"
        ],
        "in 1 month": [
            "next month"
        ],
        "in 1 week": [
            "next week"
        ],
        "in 1 year": [
            "next year"
        ]
    },
    "locale_specific": {},
    "skip": [
        " ",
        "'",
        ",",
        "-",
        ".",
        "/",
        ";",
        "@",
        "[",
        "]",
        "|",
        "，"
    ]
}



info = {
    "name": "mg",
    "date_order": "YMD",
    "january": [
        "jan",
        "janoary"
    ],
    "february": [
        "feb",
        "febroary"
    ],
    "march": [
        "mar",
        "martsa"
    ],
    "april": [
        "apr",
        "aprily"
    ],
    "may": [
        "mey"
    ],
    "june": [
        "jon",
        "jona"
    ],
    "july": [
        "jol",
        "jolay"
    ],
    "august": [
        "aog",
        "aogositra"
    ],
    "september": [
        "sep",
        "septambra"
    ],
    "october": [
        "okt",
        "oktobra"
    ],
    "november": [
        "nov",
        "novambra"
    ],
    "december": [
        "des",
        "desambra"
    ],
    "monday": [
        "alats",
        "alatsinainy"
    ],
    "tuesday": [
        "tal",
        "talata"
    ],
    "wednesday": [
        "alar",
        "alarobia"
    ],
    "thursday": [
        "alak",
        "alakamisy"
    ],
    "friday": [
        "zom",
        "zoma"
    ],
    "saturday": [
        "asab",
        "asabotsy"
    ],
    "sunday": [
        "alah",
        "alahady"
    ],
    "am": [
        "am"
    ],
    "pm": [
        "pm"
    ],
    "year": [
        "taona"
    ],
    "month": [
        "volana"
    ],
    "week": [
        "herinandro"
    ],
    "day": [
        "andro"
    ],
    "hour": [
        "ora"
    ],
    "minute": [
        "minitra"
    ],
    "second": [
        "segondra"
    ],
    "relative-type": {
        "0 day ago": [
            "anio"
        ],
        "0 hour ago": [
            "this hour"
        ],
        "0 minute ago": [
            "this minute"
        ],
        "0 month ago": [
            "this month"
        ],
        "0 second ago": [
            "now"
        ],
        "0 week ago": [
            "this week"
        ],
        "0 year ago": [
            "this year"
        ],
        "1 day ago": [
            "omaly"
        ],
        "1 month ago": [
            "last month"
        ],
        "1 week ago": [
            "last week"
        ],
        "1 year ago": [
            "last year"
        ],
        "in 1 day": [
            "rahampitso"
        ],
        "in 1 month": [
            "next month"
        ],
        "in 1 week": [
            "next week"
        ],
        "in 1 year": [
            "next year"
        ]
    },
    "locale_specific": {},
    "skip": [
        " ",
        "'",
        ",",
        "-",
        ".",
        "/",
        ";",
        "@",
        "[",
        "]",
        "|",
        "，"
    ]
}



from collections.abc import Set

from dateparser.languages.loader import LocaleDataLoader
from dateparser.conf import apply_settings, check_settings, Settings
from dateparser.date import DateDataParser
from dateparser.search.text_detection import FullTextLanguageDetector
from dateparser.custom_language_detection.language_mapping import map_languages
import regex as re


RELATIVE_REG = re.compile("(ago|in|from now|tomorrow|today|yesterday)")


def date_is_relative(translation):
    return re.search(RELATIVE_REG, translation) is not None


class _ExactLanguageSearch:
    def __init__(self, loader):
        self.loader = loader
        self.language = None

    def get_current_language(self, shortname):
        if self.language is None or self.language.shortname != shortname:
            self.language = self.loader.get_locale(shortname)

    def search(self, shortname, text, settings):
        self.get_current_language(shortname)
        result = self.language.translate_search(text, settings=settings)
        return result

    @staticmethod
    def set_relative_base(substring, already_parsed):
        if len(already_parsed) == 0:
            return substring, None

        i = len(already_parsed) - 1
        while already_parsed[i][1]:
            i -= 1
            if i == -1:
                return substring, None
        relative_base = already_parsed[i][0]['date_obj']
        return substring, relative_base

    def choose_best_split(self, possible_parsed_splits, possible_substrings_splits):
        rating = []
        for i in range(len(possible_parsed_splits)):
            num_substrings = len(possible_substrings_splits[i])
            num_substrings_without_digits = 0
            not_parsed = 0
            for j, item in enumerate(possible_parsed_splits[i]):
                if item[0]['date_obj'] is None:
                    not_parsed += 1
                if not any(char.isdigit() for char in possible_substrings_splits[i][j]):
                    num_substrings_without_digits += 1
            rating.append([
                num_substrings,
                0 if not_parsed == 0 else (float(not_parsed) / float(num_substrings)),
                0 if num_substrings_without_digits == 0 else (
                    float(num_substrings_without_digits) / float(num_substrings))])
            best_index, best_rating = min(enumerate(rating), key=lambda p: (p[1][1], p[1][0], p[1][2]))
        return possible_parsed_splits[best_index], possible_substrings_splits[best_index]

    def split_by(self, item, original, splitter):
        if item.count(splitter) <= 2:
            return [[item.split(splitter), original.split(splitter)]]

        item_all_split = item.split(splitter)
        original_all_split = original.split(splitter)
        all_possible_splits = [[item_all_split, original_all_split]]
        for i in range(2, 4):
            item_partially_split = []
            original_partially_split = []
            for j in range(0, len(item_all_split), i):
                item_join = splitter.join(item_all_split[j:j + i])
                original_join = splitter.join(original_all_split[j:j + i])
                item_partially_split.append(item_join)
                original_partially_split.append(original_join)
            all_possible_splits.append([item_partially_split, original_partially_split])
        return all_possible_splits

    def split_if_not_parsed(self, item, original):
        splitters = [',', '،', '——', '—', '–', '.', ' ']
        possible_splits = []
        for splitter in splitters:
            if splitter in item and item.count(splitter) == original.count(splitter):
                possible_splits.extend(self.split_by(item, original, splitter))
        return possible_splits

    def parse_item(self, parser, item, translated_item, parsed, need_relative_base):
        relative_base = None
        item = item.replace('ngày', '')
        item = item.replace('am', '')
        parsed_item = parser.get_date_data(item)
        is_relative = date_is_relative(translated_item)

        if need_relative_base:
            item, relative_base = self.set_relative_base(item, parsed)

        if relative_base:
            parser._settings.RELATIVE_BASE = relative_base
            parsed_item = parser.get_date_data(item)
        return parsed_item, is_relative

    def parse_found_objects(self, parser, to_parse, original, translated, settings):
        parsed = []
        substrings = []
        need_relative_base = True
        if settings.RELATIVE_BASE:
            need_relative_base = False
        for i, item in enumerate(to_parse):
            if len(item) <= 2:
                continue

            parsed_item, is_relative = self.parse_item(parser, item, translated[i], parsed, need_relative_base)
            if parsed_item['date_obj']:
                parsed.append((parsed_item, is_relative))
                substrings.append(original[i].strip(" .,:()[]-'"))
                continue

            possible_splits = self.split_if_not_parsed(item, original[i])
            if not possible_splits:
                continue

            possible_parsed = []
            possible_substrings = []
            for split_translated, split_original in possible_splits:
                current_parsed = []
                current_substrings = []
                if split_translated:
                    for j, jtem in enumerate(split_translated):
                        if len(jtem) <= 2:
                            continue
                        parsed_jtem, is_relative_jtem = self.parse_item(
                            parser, jtem, split_translated[j], current_parsed, need_relative_base)
                        current_parsed.append((parsed_jtem, is_relative_jtem))
                        current_substrings.append(split_original[j].strip(' .,:()[]-'))
                possible_parsed.append(current_parsed)
                possible_substrings.append(current_substrings)
            parsed_best, substrings_best = self.choose_best_split(possible_parsed, possible_substrings)
            for k in range(len(parsed_best)):
                if parsed_best[k][0]['date_obj']:
                    parsed.append(parsed_best[k])
                    substrings.append(substrings_best[k])
        return parsed, substrings

    def search_parse(self, shortname, text, settings):
        translated, original = self.search(shortname, text, settings)
        bad_translate_with_search = ['vi', 'hu']   # splitting done by spaces and some dictionary items contain spaces
        if shortname not in bad_translate_with_search:
            languages = ['en']
            to_parse = translated
        else:
            languages = [shortname]
            to_parse = original

        parser = DateDataParser(languages=languages, settings=settings)
        parsed, substrings = self.parse_found_objects(parser=parser, to_parse=to_parse,
                                                      original=original, translated=translated, settings=settings)
        parser._settings = Settings()
        return list(zip(substrings, [i[0]['date_obj'] for i in parsed]))


class DateSearchWithDetection:
    """
    Class which executes language detection of string in a natural language, translation of a given string,
    search of substrings which represent date and/or time and parsing of these substrings.

    """
    def __init__(self):
        self.loader = LocaleDataLoader()
        self.available_language_map = self.loader.get_locale_map()
        self.search = _ExactLanguageSearch(self.loader)

    @apply_settings
    def detect_language(self, text, languages, settings=None, detect_languages_function=None):
        if detect_languages_function and not languages:
            detected_languages = detect_languages_function(
                text, confidence_threshold=settings.LANGUAGE_DETECTION_CONFIDENCE_THRESHOLD
            )
            detected_languages = map_languages(detected_languages) or settings.DEFAULT_LANGUAGES
            return detected_languages[0] if detected_languages else None

        if isinstance(languages, (list, tuple, Set)):
            if all([language in self.available_language_map for language in languages]):
                languages = [self.available_language_map[language] for language in languages]
            else:
                unsupported_languages = set(languages) - set(self.available_language_map.keys())
                raise ValueError("Unknown language(s): %s" % ', '.join(map(repr, unsupported_languages)))
        elif languages is not None:
            raise TypeError("languages argument must be a list (%r given)" % type(languages))

        if languages:
            self.language_detector = FullTextLanguageDetector(languages=languages)
        else:
            self.language_detector = FullTextLanguageDetector(list(self.available_language_map.values()))

        detected_language = self.language_detector._best_language(text) or (
            settings.DEFAULT_LANGUAGES[0] if settings.DEFAULT_LANGUAGES else None
        )
        return detected_language

    @apply_settings
    def search_dates(self, text, languages=None, settings=None, detect_languages_function=None):
        """
        Find all substrings of the given string which represent date and/or time and parse them.

        :param text:
            A string in a natural language which may contain date and/or time expressions.
        :type text: str

        :param languages:
            A list of two letters language codes.e.g. ['en', 'es']. If languages are given, it will not attempt
            to detect the language.
        :type languages: list

        :param settings:
               Configure customized behavior using settings defined in :mod:`dateparser.conf.Settings`.
        :type settings: dict

        :param detect_languages_function:
               A function for language detection that takes as input a `text` and a `confidence_threshold`,
               returns a list of detected language codes.
        :type detect_languages_function: function

        :return: a dict mapping keys to two letter language code and a list of tuples of pairs:
                substring representing date expressions and corresponding :mod:`datetime.datetime` object.
            For example:
            {'Language': 'en', 'Dates': [('on 4 October 1957', datetime.datetime(1957, 10, 4, 0, 0))]}
            If language of the string isn't recognised returns:
            {'Language': None, 'Dates': None}
        :raises: ValueError - Unknown Language
        """

        check_settings(settings)

        language_shortname = self.detect_language(
            text=text, languages=languages, settings=settings, detect_languages_function=detect_languages_function
        )
        if not language_shortname:
            return {'Language': None, 'Dates': None}
        return {'Language': language_shortname, 'Dates': self.search.search_parse(language_shortname, text,
                                                                                  settings=settings)}



info = {
    "name": "brx",
    "date_order": "MDY",
    "january": [
        "जानुवारी"
    ],
    "february": [
        "फेब्रुवारी"
    ],
    "march": [
        "मार्स"
    ],
    "april": [
        "एफ्रिल"
    ],
    "may": [
        "मे"
    ],
    "june": [
        "जुन"
    ],
    "july": [
        "जुलाइ"
    ],
    "august": [
        "आगस्थ"
    ],
    "september": [
        "सेबथेज्ब़र"
    ],
    "october": [
        "अखथबर"
    ],
    "november": [
        "नबेज्ब़र"
    ],
    "december": [
        "दिसेज्ब़र"
    ],
    "monday": [
        "सम",
        "समबार"
    ],
    "tuesday": [
        "मंगल",
        "मंगलबार"
    ],
    "wednesday": [
        "बुद",
        "बुदबार"
    ],
    "thursday": [
        "बिसथि",
        "बिसथिबार"
    ],
    "friday": [
        "सुखुर",
        "सुखुरबार"
    ],
    "saturday": [
        "सुनि",
        "सुनिबार"
    ],
    "sunday": [
        "रबि",
        "रबिबार"
    ],
    "am": [
        "फुं"
    ],
    "pm": [
        "बेलासे"
    ],
    "year": [
        "बोसोर"
    ],
    "month": [
        "दान"
    ],
    "week": [
        "सबथा/हबथा"
    ],
    "day": [
        "सान"
    ],
    "hour": [
        "रिंगा"
    ],
    "minute": [
        "मिनिथ"
    ],
    "second": [
        "सेखेन्द"
    ],
    "relative-type": {
        "0 day ago": [
            "दिनै"
        ],
        "0 hour ago": [
            "this hour"
        ],
        "0 minute ago": [
            "this minute"
        ],
        "0 month ago": [
            "this month"
        ],
        "0 second ago": [
            "now"
        ],
        "0 week ago": [
            "this week"
        ],
        "0 year ago": [
            "this year"
        ],
        "1 day ago": [
            "मैया"
        ],
        "1 month ago": [
            "last month"
        ],
        "1 week ago": [
            "last week"
        ],
        "1 year ago": [
            "last year"
        ],
        "in 1 day": [
            "गाबोन"
        ],
        "in 1 month": [
            "next month"
        ],
        "in 1 week": [
            "next week"
        ],
        "in 1 year": [
            "next year"
        ]
    },
    "locale_specific": {},
    "skip": [
        " ",
        "'",
        ",",
        "-",
        ".",
        "/",
        ";",
        "@",
        "[",
        "]",
        "|",
        "，"
    ]
}



info = {
    "name": "jgo",
    "date_order": "YMD",
    "january": [
        "nduŋmbi saŋ"
    ],
    "february": [
        "pɛsaŋ pɛ́pá"
    ],
    "march": [
        "pɛsaŋ pɛ́tát"
    ],
    "april": [
        "pɛsaŋ pɛ́nɛ́kwa"
    ],
    "may": [
        "pɛsaŋ pataa"
    ],
    "june": [
        "pɛsaŋ pɛ́nɛ́ntúkú"
    ],
    "july": [
        "pɛsaŋ saambá"
    ],
    "august": [
        "pɛsaŋ pɛ́nɛ́fɔm"
    ],
    "september": [
        "pɛsaŋ pɛ́nɛ́pfúꞌú"
    ],
    "october": [
        "pɛsaŋ nɛgɛ́m"
    ],
    "november": [
        "pɛsaŋ ntsɔ̌pmɔ́"
    ],
    "december": [
        "pɛsaŋ ntsɔ̌ppá"
    ],
    "monday": [
        "mɔ́ndi"
    ],
    "tuesday": [
        "ápta mɔ́ndi"
    ],
    "wednesday": [
        "wɛ́nɛsɛdɛ"
    ],
    "thursday": [
        "tɔ́sɛdɛ"
    ],
    "friday": [
        "fɛlâyɛdɛ"
    ],
    "saturday": [
        "sásidɛ"
    ],
    "sunday": [
        "sɔ́ndi"
    ],
    "am": [
        "mba'mba'"
    ],
    "pm": [
        "ŋka mbɔ́t nji"
    ],
    "year": [
        "year"
    ],
    "month": [
        "month"
    ],
    "week": [
        "week"
    ],
    "day": [
        "day"
    ],
    "hour": [
        "hour"
    ],
    "minute": [
        "minute"
    ],
    "second": [
        "second"
    ],
    "relative-type": {
        "0 day ago": [
            "lɔꞌɔ"
        ],
        "0 hour ago": [
            "this hour"
        ],
        "0 minute ago": [
            "this minute"
        ],
        "0 month ago": [
            "this month"
        ],
        "0 second ago": [
            "now"
        ],
        "0 week ago": [
            "this week"
        ],
        "0 year ago": [
            "this year"
        ],
        "1 day ago": [
            "yesterday"
        ],
        "1 month ago": [
            "last month"
        ],
        "1 week ago": [
            "last week"
        ],
        "1 year ago": [
            "last year"
        ],
        "in 1 day": [
            "tomorrow"
        ],
        "in 1 month": [
            "next month"
        ],
        "in 1 week": [
            "next week"
        ],
        "in 1 year": [
            "next year"
        ]
    },
    "relative-type-regex": {
        "\\1 day ago": [
            "ɛ́ gɛ́ mɔ́ lɛ́ꞌ (\\d+[.,]?\\d*)"
        ],
        "\\1 hour ago": [
            "ɛ́ gɛ mɔ́ (\\d+[.,]?\\d*) háwa"
        ],
        "\\1 minute ago": [
            "ɛ́ gɛ́ mɔ́ minút (\\d+[.,]?\\d*)"
        ],
        "\\1 month ago": [
            "ɛ́ gɛ́ mɔ́ pɛsaŋ (\\d+[.,]?\\d*)"
        ],
        "\\1 week ago": [
            "ɛ́ gɛ́ mɔ (\\d+[.,]?\\d*) ŋgap-mbi"
        ],
        "\\1 year ago": [
            "ɛ́gɛ́ mɔ́ ŋguꞌ (\\d+[.,]?\\d*)"
        ],
        "in \\1 day": [
            "nǔu lɛ́ꞌ (\\d+[.,]?\\d*)"
        ],
        "in \\1 hour": [
            "nǔu háwa (\\d+[.,]?\\d*)"
        ],
        "in \\1 minute": [
            "nǔu (\\d+[.,]?\\d*) minút"
        ],
        "in \\1 month": [
            "nǔu (\\d+[.,]?\\d*) saŋ"
        ],
        "in \\1 week": [
            "nǔu ŋgap-mbi (\\d+[.,]?\\d*)"
        ],
        "in \\1 year": [
            "nǔu ŋguꞌ (\\d+[.,]?\\d*)"
        ]
    },
    "locale_specific": {},
    "skip": [
        " ",
        "'",
        ",",
        "-",
        ".",
        "/",
        ";",
        "@",
        "[",
        "]",
        "|",
        "，"
    ]
}



info = {
    "sentence_splitter_group": 1,
    "skip": [
        "ganap",
        "na",
        "noon",
        "noong",
        "sa",
        " ",
        "'",
        ",",
        "-",
        ".",
        "/",
        ";",
        "@",
        "[",
        "]",
        "|",
        "，"
    ],
    "monday": [
        "Lunes",
        "Lun"
    ],
    "tuesday": [
        "Martes"
    ],
    "wednesday": [
        "Miyerkules",
        "Miy"
    ],
    "thursday": [
        "Huwebes",
        "Huw"
    ],
    "friday": [
        "Biyernes",
        "Biy"
    ],
    "saturday": [
        "Sabado",
        "Sab"
    ],
    "sunday": [
        "Linggo",
        "Lin"
    ],
    "january": [
        "Enero",
        "Ene"
    ],
    "february": [
        "Pebrero",
        "Peb"
    ],
    "march": [
        "Marso",
        "Mar"
    ],
    "april": [
        "Abril",
        "Abr"
    ],
    "may": [
        "Mayo",
        "May"
    ],
    "june": [
        "Hunyo",
        "Hun"
    ],
    "july": [
        "Hulyo",
        "Hul"
    ],
    "august": [
        "Agosto",
        "Ago"
    ],
    "september": [
        "Setyembre",
        "Set"
    ],
    "october": [
        "Oktubre",
        "Okt"
    ],
    "november": [
        "Nobyembre",
        "Nob"
    ],
    "december": [
        "Disyembre",
        "Dis"
    ],
    "year": [
        "taon"
    ],
    "month": [
        "buwan"
    ],
    "week": [
        "linggo"
    ],
    "day": [
        "araw"
    ],
    "hour": [
        "oras"
    ],
    "minute": [
        "minuto"
    ],
    "second": [
        "segundo"
    ],
    "ago": [
        "nakaraan",
        "nakalipas"
    ],
    "in": [
        "sa"
    ],
    "simplifications": [
        {
            "kahapon": "1 araw nakaraan"
        },
        {
            "ngayon": "0 segundo nakalipas"
        },
        {
            "isang araw": "2 araw"
        }
    ],
    "name": "tl"
}



info = {
    "name": "ms",
    "date_order": "DMY",
    "january": [
        "jan",
        "januari"
    ],
    "february": [
        "feb",
        "februari"
    ],
    "march": [
        "mac"
    ],
    "april": [
        "apr",
        "april"
    ],
    "may": [
        "mei"
    ],
    "june": [
        "jun"
    ],
    "july": [
        "jul",
        "julai"
    ],
    "august": [
        "ogo",
        "ogos"
    ],
    "september": [
        "sep",
        "september"
    ],
    "october": [
        "okt",
        "oktober"
    ],
    "november": [
        "nov",
        "november"
    ],
    "december": [
        "dis",
        "disember"
    ],
    "monday": [
        "isn",
        "isnin"
    ],
    "tuesday": [
        "sel",
        "selasa"
    ],
    "wednesday": [
        "rab",
        "rabu"
    ],
    "thursday": [
        "kha",
        "khamis"
    ],
    "friday": [
        "jum",
        "jumaat"
    ],
    "saturday": [
        "sab",
        "sabtu"
    ],
    "sunday": [
        "ahad",
        "ahd"
    ],
    "am": [
        "pg"
    ],
    "pm": [
        "ptg"
    ],
    "year": [
        "tahun",
        "thn"
    ],
    "month": [
        "bln",
        "bulan"
    ],
    "week": [
        "mgu",
        "minggu"
    ],
    "day": [
        "hari"
    ],
    "hour": [
        "jam"
    ],
    "minute": [
        "min",
        "minit"
    ],
    "second": [
        "saat"
    ],
    "relative-type": {
        "0 day ago": [
            "hari ini"
        ],
        "0 hour ago": [
            "jam ini"
        ],
        "0 minute ago": [
            "pada minit ini"
        ],
        "0 month ago": [
            "bln ini",
            "bulan ini"
        ],
        "0 second ago": [
            "sekarang"
        ],
        "0 week ago": [
            "minggu ini",
            "mng ini"
        ],
        "0 year ago": [
            "tahun ini",
            "thn ini"
        ],
        "1 day ago": [
            "semalam",
            "semlm"
        ],
        "1 month ago": [
            "bln lalu",
            "bulan lalu"
        ],
        "1 week ago": [
            "minggu lalu",
            "mng lepas"
        ],
        "1 year ago": [
            "tahun lalu",
            "thn lepas"
        ],
        "in 1 day": [
            "esok"
        ],
        "in 1 month": [
            "bln depan",
            "bulan depan"
        ],
        "in 1 week": [
            "minggu depan",
            "mng depan"
        ],
        "in 1 year": [
            "tahun depan",
            "thn depan"
        ]
    },
    "relative-type-regex": {
        "\\1 day ago": [
            "(\\d+[.,]?\\d*) hari lalu"
        ],
        "\\1 hour ago": [
            "(\\d+[.,]?\\d*) jam lalu"
        ],
        "\\1 minute ago": [
            "(\\d+[.,]?\\d*) min lalu",
            "(\\d+[.,]?\\d*) minit lalu"
        ],
        "\\1 month ago": [
            "(\\d+[.,]?\\d*) bln lalu",
            "(\\d+[.,]?\\d*) bulan lalu"
        ],
        "\\1 second ago": [
            "(\\d+[.,]?\\d*) saat lalu"
        ],
        "\\1 week ago": [
            "(\\d+[.,]?\\d*) mgu lalu",
            "(\\d+[.,]?\\d*) minggu lalu"
        ],
        "\\1 year ago": [
            "(\\d+[.,]?\\d*) tahun lalu",
            "(\\d+[.,]?\\d*) thn lalu"
        ],
        "in \\1 day": [
            "dalam (\\d+[.,]?\\d*) hari",
            "dlm (\\d+[.,]?\\d*) hari"
        ],
        "in \\1 hour": [
            "dalam (\\d+[.,]?\\d*) jam",
            "dlm (\\d+[.,]?\\d*) jam"
        ],
        "in \\1 minute": [
            "dalam (\\d+[.,]?\\d*) minit",
            "dlm (\\d+[.,]?\\d*) min"
        ],
        "in \\1 month": [
            "dalam (\\d+[.,]?\\d*) bulan",
            "dlm (\\d+[.,]?\\d*) bln"
        ],
        "in \\1 second": [
            "dalam (\\d+[.,]?\\d*) saat",
            "dlm (\\d+[.,]?\\d*) saat"
        ],
        "in \\1 week": [
            "dalam (\\d+[.,]?\\d*) minggu",
            "dlm (\\d+[.,]?\\d*) mgu"
        ],
        "in \\1 year": [
            "dalam (\\d+[.,]?\\d*) saat",
            "dalam (\\d+[.,]?\\d*) thn"
        ]
    },
    "locale_specific": {
        "ms-BN": {
            "name": "ms-BN"
        },
        "ms-SG": {
            "name": "ms-SG"
        }
    },
    "skip": [
        " ",
        "'",
        ",",
        "-",
        ".",
        "/",
        ";",
        "@",
        "[",
        "]",
        "|",
        "，"
    ]
}



info = {
    "name": "saq",
    "date_order": "DMY",
    "january": [
        "lapa le obo",
        "obo"
    ],
    "february": [
        "lapa le waare",
        "waa"
    ],
    "march": [
        "lapa le okuni",
        "oku"
    ],
    "april": [
        "lapa le ong'wan",
        "ong"
    ],
    "may": [
        "ime",
        "lapa le imet"
    ],
    "june": [
        "ile",
        "lapa le ile"
    ],
    "july": [
        "lapa le sapa",
        "sap"
    ],
    "august": [
        "isi",
        "lapa le isiet"
    ],
    "september": [
        "lapa le saal",
        "saa"
    ],
    "october": [
        "lapa le tomon",
        "tom"
    ],
    "november": [
        "lapa le tomon obo",
        "tob"
    ],
    "december": [
        "lapa le tomon waare",
        "tow"
    ],
    "monday": [
        "kun",
        "mderot ee kuni"
    ],
    "tuesday": [
        "mderot ee ong'wan",
        "ong"
    ],
    "wednesday": [
        "ine",
        "mderot ee inet"
    ],
    "thursday": [
        "ile",
        "mderot ee ile"
    ],
    "friday": [
        "mderot ee sapa",
        "sap"
    ],
    "saturday": [
        "kwe",
        "mderot ee kwe"
    ],
    "sunday": [
        "are",
        "mderot ee are"
    ],
    "am": [
        "tesiran"
    ],
    "pm": [
        "teipa"
    ],
    "year": [
        "lari"
    ],
    "month": [
        "lapa"
    ],
    "week": [
        "saipa napo"
    ],
    "day": [
        "mpari"
    ],
    "hour": [
        "saai"
    ],
    "minute": [
        "idakika"
    ],
    "second": [
        "isekondi"
    ],
    "relative-type": {
        "0 day ago": [
            "duo"
        ],
        "0 hour ago": [
            "this hour"
        ],
        "0 minute ago": [
            "this minute"
        ],
        "0 month ago": [
            "this month"
        ],
        "0 second ago": [
            "now"
        ],
        "0 week ago": [
            "this week"
        ],
        "0 year ago": [
            "this year"
        ],
        "1 day ago": [
            "ng'ole"
        ],
        "1 month ago": [
            "last month"
        ],
        "1 week ago": [
            "last week"
        ],
        "1 year ago": [
            "last year"
        ],
        "in 1 day": [
            "taisere"
        ],
        "in 1 month": [
            "next month"
        ],
        "in 1 week": [
            "next week"
        ],
        "in 1 year": [
            "next year"
        ]
    },
    "locale_specific": {},
    "skip": [
        " ",
        "'",
        ",",
        "-",
        ".",
        "/",
        ";",
        "@",
        "[",
        "]",
        "|",
        "，"
    ]
}



info = {
    "name": "to",
    "date_order": "DMY",
    "january": [
        "sān",
        "sānuali"
    ],
    "february": [
        "fēp",
        "fēpueli"
    ],
    "march": [
        "ma'a",
        "ma'asi"
    ],
    "april": [
        "'epe",
        "'epeleli"
    ],
    "may": [
        "mē"
    ],
    "june": [
        "sun",
        "sune"
    ],
    "july": [
        "siu",
        "siulai"
    ],
    "august": [
        "'aok",
        "'aokosi"
    ],
    "september": [
        "sep",
        "sepitema"
    ],
    "october": [
        "'oka",
        "'okatopa"
    ],
    "november": [
        "nōv",
        "nōvema"
    ],
    "december": [
        "tīs",
        "tīsema"
    ],
    "monday": [
        "mōn",
        "mōnite"
    ],
    "tuesday": [
        "tūs",
        "tūsite"
    ],
    "wednesday": [
        "pul",
        "pulelulu"
    ],
    "thursday": [
        "tu'a",
        "tu'apulelulu"
    ],
    "friday": [
        "fal",
        "falaite"
    ],
    "saturday": [
        "tok",
        "tokonaki"
    ],
    "sunday": [
        "sāp",
        "sāpate"
    ],
    "am": [
        "am",
        "hengihengi",
        "hh"
    ],
    "pm": [
        "ea",
        "efiafi",
        "pm"
    ],
    "year": [
        "ta'u"
    ],
    "month": [
        "māhina"
    ],
    "week": [
        "uike"
    ],
    "day": [
        "'aho"
    ],
    "hour": [
        "houa"
    ],
    "minute": [
        "miniti"
    ],
    "second": [
        "sekoni"
    ],
    "relative-type": {
        "0 day ago": [
            "'ahó ni"
        ],
        "0 hour ago": [
            "this hour"
        ],
        "0 minute ago": [
            "this minute"
        ],
        "0 month ago": [
            "māhiná ni"
        ],
        "0 second ago": [
            "taimí ni"
        ],
        "0 week ago": [
            "uiké ni"
        ],
        "0 year ago": [
            "ta'ú ni"
        ],
        "1 day ago": [
            "'aneafi"
        ],
        "1 month ago": [
            "māhina kuo'osi"
        ],
        "1 week ago": [
            "uike kuo'osi"
        ],
        "1 year ago": [
            "ta'u kuo'osi"
        ],
        "in 1 day": [
            "'apongipongi"
        ],
        "in 1 month": [
            "māhina kaha'u"
        ],
        "in 1 week": [
            "uike kaha'u"
        ],
        "in 1 year": [
            "ta'u kaha'u"
        ]
    },
    "relative-type-regex": {
        "\\1 day ago": [
            "'aho 'e (\\d+[.,]?\\d*) kuo'osi"
        ],
        "\\1 hour ago": [
            "houa 'e (\\d+[.,]?\\d*) kuo'osi"
        ],
        "\\1 minute ago": [
            "miniti 'e (\\d+[.,]?\\d*) kuo'osi"
        ],
        "\\1 month ago": [
            "māhina 'e (\\d+[.,]?\\d*) kuo'osi"
        ],
        "\\1 second ago": [
            "sekoni 'e (\\d+[.,]?\\d*) kuo'osi"
        ],
        "\\1 week ago": [
            "uike 'e (\\d+[.,]?\\d*) kuo'osi"
        ],
        "\\1 year ago": [
            "ta'u 'e (\\d+[.,]?\\d*) kuo'osi"
        ],
        "in \\1 day": [
            "'i he 'aho 'e (\\d+[.,]?\\d*)"
        ],
        "in \\1 hour": [
            "'i he houa 'e (\\d+[.,]?\\d*)"
        ],
        "in \\1 minute": [
            "'i he miniti 'e (\\d+[.,]?\\d*)"
        ],
        "in \\1 month": [
            "'i he māhina 'e (\\d+[.,]?\\d*)"
        ],
        "in \\1 second": [
            "'i he sekoni 'e (\\d+[.,]?\\d*)"
        ],
        "in \\1 week": [
            "'i he uike 'e (\\d+[.,]?\\d*)"
        ],
        "in \\1 year": [
            "'i he ta'u 'e (\\d+[.,]?\\d*)"
        ]
    },
    "locale_specific": {},
    "skip": [
        " ",
        "'",
        ",",
        "-",
        ".",
        "/",
        ";",
        "@",
        "[",
        "]",
        "|",
        "，"
    ]
}



from functools import wraps


def _restore_languages_on_generator_exit(method):
    @wraps(method)
    def wrapped(self, *args, **kwargs):
        stored_languages = self.languages[:]
        for language in method(self, *args, **kwargs):
            yield language
        else:
            self.languages[:] = stored_languages

    return wrapped


class BaseLanguageDetector:
    def __init__(self, languages):
        self.languages = languages[:]

    @_restore_languages_on_generator_exit
    def iterate_applicable_languages(self, date_string, settings=None, modify=False):
        languages = self.languages if modify else self.languages[:]
        yield from self._filter_languages(date_string, languages, settings)

    @staticmethod
    def _filter_languages(date_string, languages, settings=None):
        while languages:
            language = languages[0]
            if language.is_applicable(date_string, strip_timezone=False, settings=settings):
                yield language
            elif language.is_applicable(date_string, strip_timezone=True, settings=settings):
                yield language

            languages.pop(0)


class AutoDetectLanguage(BaseLanguageDetector):
    def __init__(self, languages, allow_redetection=False):
        super().__init__(languages=languages[:])
        self.language_pool = languages[:]
        self.allow_redetection = allow_redetection

    @_restore_languages_on_generator_exit
    def iterate_applicable_languages(self, date_string, modify=False, settings=None):
        languages = self.languages if modify else self.languages[:]
        initial_languages = languages[:]
        yield from self._filter_languages(date_string, languages, settings=settings)

        if not self.allow_redetection:
            return

        # Try languages that was not tried before with this date_string
        languages = [language
                     for language in self.language_pool
                     if language not in initial_languages]
        if modify:
            self.languages = languages

        yield from self._filter_languages(date_string, languages, settings=settings)


class ExactLanguages(BaseLanguageDetector):
    def __init__(self, languages):
        if languages is None:
            raise ValueError("language cannot be None for ExactLanguages")
        super().__init__(languages=languages)

    @_restore_languages_on_generator_exit
    def iterate_applicable_languages(self, date_string, modify=False, settings=None):
        yield from super().iterate_applicable_languages(date_string, modify=False, settings=settings)



info = {
    "name": "rm",
    "date_order": "DMY",
    "january": [
        "schan",
        "schaner"
    ],
    "february": [
        "favr",
        "favrer"
    ],
    "march": [
        "mars"
    ],
    "april": [
        "avr",
        "avrigl"
    ],
    "may": [
        "matg"
    ],
    "june": [
        "zercl",
        "zercladur"
    ],
    "july": [
        "fan",
        "fanadur"
    ],
    "august": [
        "avust"
    ],
    "september": [
        "sett",
        "settember"
    ],
    "october": [
        "oct",
        "october"
    ],
    "november": [
        "nov",
        "november"
    ],
    "december": [
        "dec",
        "december"
    ],
    "monday": [
        "gli",
        "glindesdi"
    ],
    "tuesday": [
        "ma",
        "mardi"
    ],
    "wednesday": [
        "me",
        "mesemna"
    ],
    "thursday": [
        "gie",
        "gievgia"
    ],
    "friday": [
        "ve",
        "venderdi"
    ],
    "saturday": [
        "so",
        "sonda"
    ],
    "sunday": [
        "du",
        "dumengia"
    ],
    "am": [
        "am"
    ],
    "pm": [
        "pm"
    ],
    "year": [
        "onn"
    ],
    "month": [
        "mais"
    ],
    "week": [
        "emna"
    ],
    "day": [
        "tag"
    ],
    "hour": [
        "ura"
    ],
    "minute": [
        "minuta"
    ],
    "second": [
        "secunda"
    ],
    "relative-type": {
        "0 day ago": [
            "oz"
        ],
        "0 hour ago": [
            "this hour"
        ],
        "0 minute ago": [
            "this minute"
        ],
        "0 month ago": [
            "this month"
        ],
        "0 second ago": [
            "now"
        ],
        "0 week ago": [
            "this week"
        ],
        "0 year ago": [
            "this year"
        ],
        "1 day ago": [
            "ier"
        ],
        "1 month ago": [
            "last month"
        ],
        "1 week ago": [
            "last week"
        ],
        "1 year ago": [
            "last year"
        ],
        "in 1 day": [
            "damaun"
        ],
        "in 1 month": [
            "next month"
        ],
        "in 1 week": [
            "next week"
        ],
        "in 1 year": [
            "next year"
        ]
    },
    "locale_specific": {},
    "skip": [
        " ",
        "'",
        ",",
        "-",
        ".",
        "/",
        ";",
        "@",
        "[",
        "]",
        "|",
        "，"
    ]
}



import sys
import importlib.util
import regex as re

from datetime import datetime

TIME_MATCHER = re.compile(
    r'.*?'
    r'(?P<hour>2[0-3]|[0-1]\d|\d):'
    r'(?P<minute>[0-5]\d|\d):'
    r'(?P<second>6[0-1]|[0-5]\d|\d)'
    r'\.(?P<microsecond>[0-9]{1,6})'
)

MS_SEARCHER = re.compile(r'\.(?P<microsecond>[0-9]{1,6})')


def patch_strptime():
    """Monkey patching _strptime to avoid problems related with non-english
    locale changes on the system.

    For example, if system's locale is set to fr_FR. Parser won't recognize
    any date since all languages are translated to english dates.
    """
    _strptime_spec = importlib.util.find_spec('_strptime')

    _strptime = importlib.util.module_from_spec(_strptime_spec)
    _strptime_spec.loader.exec_module(_strptime)
    sys.modules['strptime_patched'] = _strptime

    _calendar = importlib.util.module_from_spec(_strptime_spec)
    _strptime_spec.loader.exec_module(_calendar)
    sys.modules['calendar_patched'] = _calendar

    _strptime._getlang = lambda: ('en_US', 'UTF-8')
    _strptime.calendar = _calendar
    _strptime.calendar.day_abbr = [
        'mon', 'tue', 'wed', 'thu', 'fri', 'sat', 'sun'
    ]
    _strptime.calendar.day_name = [
        'monday', 'tuesday', 'wednesday', 'thursday',
        'friday', 'saturday', 'sunday'
    ]
    _strptime.calendar.month_abbr = [
        '', 'jan', 'feb', 'mar', 'apr', 'may', 'jun',
        'jul', 'aug', 'sep', 'oct', 'nov', 'dec'
    ]
    _strptime.calendar.month_name = [
        '', 'january', 'february', 'march', 'april',
        'may', 'june', 'july', 'august', 'september',
        'october', 'november', 'december'
    ]

    return _strptime._strptime_time


__strptime = patch_strptime()


def strptime(date_string, format):
    obj = datetime(*__strptime(date_string, format)[:-3])

    if '%f' in format:
        try:
            match_groups = TIME_MATCHER.match(date_string).groupdict()
            ms = match_groups['microsecond']
            ms = ms + ((6 - len(ms)) * '0')
            obj = obj.replace(microsecond=int(ms))
        except AttributeError:
            match_groups = MS_SEARCHER.search(date_string).groupdict()
            ms = match_groups['microsecond']
            ms = ms + ((6 - len(ms)) * '0')
            obj = obj.replace(microsecond=int(ms))

    return obj



info = {
    "name": "mt",
    "date_order": "DMY",
    "january": [
        "jan",
        "jannar"
    ],
    "february": [
        "fra",
        "frar"
    ],
    "march": [
        "mar",
        "marzu"
    ],
    "april": [
        "apr",
        "april"
    ],
    "may": [
        "mej",
        "mejju"
    ],
    "june": [
        "ġun",
        "ġunju"
    ],
    "july": [
        "lul",
        "lulju"
    ],
    "august": [
        "aww",
        "awwissu"
    ],
    "september": [
        "set",
        "settembru"
    ],
    "october": [
        "ott",
        "ottubru"
    ],
    "november": [
        "nov",
        "novembru"
    ],
    "december": [
        "diċ",
        "diċembru"
    ],
    "monday": [
        "it-tnejn",
        "tne"
    ],
    "tuesday": [
        "it-tlieta",
        "tli"
    ],
    "wednesday": [
        "erb",
        "l-erbgħa"
    ],
    "thursday": [
        "il-ħamis",
        "ħam"
    ],
    "friday": [
        "il-ġimgħa",
        "ġim"
    ],
    "saturday": [
        "is-sibt",
        "sib"
    ],
    "sunday": [
        "il-ħadd",
        "ħad"
    ],
    "am": [
        "am"
    ],
    "pm": [
        "pm"
    ],
    "year": [
        "sena"
    ],
    "month": [
        "xahar"
    ],
    "week": [
        "ġimgħa"
    ],
    "day": [
        "jum"
    ],
    "hour": [
        "siegħa"
    ],
    "minute": [
        "minuta"
    ],
    "second": [
        "sekonda"
    ],
    "relative-type": {
        "0 day ago": [
            "illum"
        ],
        "0 hour ago": [
            "this hour"
        ],
        "0 minute ago": [
            "this minute"
        ],
        "0 month ago": [
            "dan ix-xahar"
        ],
        "0 second ago": [
            "now"
        ],
        "0 week ago": [
            "din il-ġimgħa"
        ],
        "0 year ago": [
            "din is-sena"
        ],
        "1 day ago": [
            "ilbieraħ"
        ],
        "1 month ago": [
            "ix-xahar li għadda"
        ],
        "1 week ago": [
            "il-ġimgħa li għaddiet"
        ],
        "1 year ago": [
            "is-sena li għaddiet"
        ],
        "in 1 day": [
            "għada"
        ],
        "in 1 month": [
            "ix-xahar id-dieħel"
        ],
        "in 1 week": [
            "il-ġimgħa d-dieħla"
        ],
        "in 1 year": [
            "is-sena d-dieħla"
        ]
    },
    "relative-type-regex": {
        "\\1 year ago": [
            "(\\d+[.,]?\\d*) sena ilu",
            "(\\d+[.,]?\\d*) snin ilu"
        ]
    },
    "locale_specific": {},
    "skip": [
        " ",
        "'",
        ",",
        "-",
        ".",
        "/",
        ";",
        "@",
        "[",
        "]",
        "|",
        "，"
    ]
}



info = {
    "name": "hr",
    "date_order": "DMY",
    "january": [
        "sij",
        "siječanj",
        "siječnja",
        "siječnju",
        "siječnjem"
    ],
    "february": [
        "velj",
        "veljača",
        "veljače",
        "veljači",
        "veljačom"
    ],
    "march": [
        "ožu",
        "ožujak",
        "ožujka",
        "ožujku",
        "ožujkom"
    ],
    "april": [
        "tra",
        "travanj",
        "travnja",
        "travnju",
        "travnjem"
    ],
    "may": [
        "svi",
        "svibanj",
        "svibnja",
        "svibnju",
        "svibnjem"
    ],
    "june": [
        "lip",
        "lipanj",
        "lipnja",
        "lipnju",
        "lipnjem"
    ],
    "july": [
        "srp",
        "srpanj",
        "srpnja",
        "srpnju",
        "srpnjem"
    ],
    "august": [
        "kol",
        "kolovoz",
        "kolovoza",
        "kolovozu",
        "kolovozom"
    ],
    "september": [
        "ruj",
        "rujan",
        "rujna",
        "rujnu",
        "rujnom"
    ],
    "october": [
        "lis",
        "listopad",
        "listopada",
        "listopadu",
        "listopadom"
    ],
    "november": [
        "stu",
        "studeni",
        "studenoga",
        "studenom",
        "studenomu",
        "studenome",
        "studenog",
        "studenim"
    ],
    "december": [
        "pro",
        "prosinac",
        "prosinca",
        "prosincu",
        "prosincem"
    ],
    "monday": [
        "pon",
        "ponedjeljak"
    ],
    "tuesday": [
        "uto",
        "utorak"
    ],
    "wednesday": [
        "sri",
        "srijeda"
    ],
    "thursday": [
        "čet",
        "četvrtak"
    ],
    "friday": [
        "pet",
        "petak"
    ],
    "saturday": [
        "sub",
        "subota"
    ],
    "sunday": [
        "ned",
        "nedjelja"
    ],
    "am": [
        "am"
    ],
    "pm": [
        "pm"
    ],
    "year": [
        "g",
        "godina"
    ],
    "month": [
        "m",
        "mj",
        "mjesec"
    ],
    "week": [
        "tj",
        "tjedan"
    ],
    "day": [
        "d",
        "dan"
    ],
    "hour": [
        "h",
        "sat"
    ],
    "minute": [
        "min",
        "minuta"
    ],
    "second": [
        "s",
        "sekunda"
    ],
    "relative-type": {
        "0 day ago": [
            "danas"
        ],
        "0 hour ago": [
            "ovaj sat"
        ],
        "0 minute ago": [
            "ova minuta"
        ],
        "0 month ago": [
            "ovaj mj",
            "ovaj mjesec"
        ],
        "0 second ago": [
            "sad"
        ],
        "0 week ago": [
            "ovaj tj",
            "ovaj tjedan"
        ],
        "0 year ago": [
            "ove g",
            "ove god",
            "ove godine"
        ],
        "1 day ago": [
            "jučer",
            "jučer"
        ],
        "1 month ago": [
            "prošli mj",
            "prošli mjesec",
            "prošlog mjeseca",
            "prošlom mjesecu"
        ],
        "1 week ago": [
            "prošli tj",
            "prošli tjedan",
            "prošlog tjedna",
            "prošlom tjednu"
        ],
        "1 year ago": [
            "prošle g",
            "prošle god",
            "prošle godine",
            "prošla godina",
            "prošloj godini",
            "lani"
        ],
        "in 1 day": [
            "sutra",
            "sutra"
        ],
        "in 1 month": [
            "sljedeći mj",
            "sljedeći mjesec",
            "sljedećeg mjeseca",
            "sljedećem mjesecu",
            "idući mj",
            "idućeg mj",
            "idućem mj",
            "idući mjesec",
            "idućeg mjeseca",
            "idućem mjesecu"
        ],
        "in 1 week": [
            "sljedeći tj",
            "sljedeći tjedan",
            "sljedećeg tjedna",
            "sljedećem tjednu",
            "idući tj",
            "idućeg tj",
            "idućem tj",
            "idući tjedan",
            "idućeg tjedna",
            "idućem tjednu"
        ],
        "in 1 year": [
            "sljedeće g",
            "sljedeće god",
            "sljedeće godine",
            "sljedeća g",
            "sljedeća god",
            "sljedeća godina",
            "sljedećoj godini",
            "iduća g",
            "iduće g",
            "idućoj g",
            "iduća god",
            "iduće god",
            "idućoj god",
            "iduća godina",
            "iduće godine",
            "idućoj godini",
            "dogodine",
            "nagodinu"
        ],
        "in 2 day": [
            "prekosutra"
        ],
        "2 year ago": [
            "preklani"
        ],
        "2 day ago": [
            "prekjučer"
        ]
    },
    "relative-type-regex": {
        "\\1 day ago": [
            "prije (\\d+[.,]?\\d*) d",
            "prije (\\d+[.,]?\\d*) dan",
            "prije (\\d+[.,]?\\d*) dana"
        ],
        "\\1 hour ago": [
            "prije (\\d+[.,]?\\d*) h",
            "prije (\\d+[.,]?\\d*) sat",
            "prije (\\d+[.,]?\\d*) sati",
            "prije (\\d+[.,]?\\d*) sata"
        ],
        "\\1 minute ago": [
            "prije (\\d+[.,]?\\d*) min",
            "prije (\\d+[.,]?\\d*) minuta",
            "prije (\\d+[.,]?\\d*) minutu",
            "prije (\\d+[.,]?\\d*) minute"
        ],
        "\\1 month ago": [
            "prije (\\d+[.,]?\\d*) mj",
            "prije (\\d+[.,]?\\d*) mjesec",
            "prije (\\d+[.,]?\\d*) mjeseci",
            "prije (\\d+[.,]?\\d*) mjeseca"
        ],
        "\\1 second ago": [
            "prije (\\d+[.,]?\\d*) s",
            "prije (\\d+[.,]?\\d*) sekundi",
            "prije (\\d+[.,]?\\d*) sekundu",
            "prije (\\d+[.,]?\\d*) sekunde",
            "prije (\\d+[.,]?\\d*) sekundu"
        ],
        "\\1 week ago": [
            "prije (\\d+[.,]?\\d*) tj",
            "prije (\\d+[.,]?\\d*) tjedan",
            "prije (\\d+[.,]?\\d*) tjedana",
            "prije (\\d+[.,]?\\d*) tjedna"
        ],
        "\\1 year ago": [
            "prije (\\d+[.,]?\\d*) g",
            "prije (\\d+[.,]?\\d*) godina",
            "prije (\\d+[.,]?\\d*) godinu",
            "prije (\\d+[.,]?\\d*) godine"
        ],
        "in \\1 day": [
            "za (\\d+[.,]?\\d*) d",
            "za (\\d+[.,]?\\d*) dan",
            "za (\\d+[.,]?\\d*) dana"
        ],
        "in \\1 hour": [
            "za (\\d+[.,]?\\d*) h",
            "za (\\d+[.,]?\\d*) sat",
            "za (\\d+[.,]?\\d*) sati",
            "za (\\d+[.,]?\\d*) sata"
        ],
        "in \\1 minute": [
            "za (\\d+[.,]?\\d*) min",
            "za (\\d+[.,]?\\d*) minuta",
            "za (\\d+[.,]?\\d*) minutu",
            "za (\\d+[.,]?\\d*) minute"
        ],
        "in \\1 month": [
            "za (\\d+[.,]?\\d*) mj",
            "za (\\d+[.,]?\\d*) mjesec",
            "za (\\d+[.,]?\\d*) mjeseci",
            "za (\\d+[.,]?\\d*) mjeseca"
        ],
        "in \\1 second": [
            "za (\\d+[.,]?\\d*) s",
            "za (\\d+[.,]?\\d*) sekundi",
            "za (\\d+[.,]?\\d*) sekundu",
            "za (\\d+[.,]?\\d*) sekunde",
            "za (\\d+[.,]?\\d*) sekundu"
        ],
        "in \\1 week": [
            "za (\\d+[.,]?\\d*) tj",
            "za (\\d+[.,]?\\d*) tjedan",
            "za (\\d+[.,]?\\d*) tjedana",
            "za (\\d+[.,]?\\d*) tjedna"
        ],
        "in \\1 year": [
            "za (\\d+[.,]?\\d*) g",
            "za (\\d+[.,]?\\d*) godina",
            "za (\\d+[.,]?\\d*) godinu",
            "za (\\d+[.,]?\\d*) godine"
        ]
    },
    "locale_specific": {
        "hr-BA": {
            "name": "hr-BA"
        }
    },
    "in": [
        "u"
    ],
    "simplifications": [
        {
            "podne": "12:00"
        },
        {
            "ponoć": "00:00"
        }
    ],
    "skip": [
        " ",
        "'",
        ",",
        "-",
        ".",
        "/",
        ";",
        "@",
        "[",
        "]",
        "|",
        "，"
    ]
}



info = {
    "name": "pl",
    "date_order": "DMY",
    "january": [
        "sty",
        "styczeń",
        "stycznia",
        "Styczeń",
        "Styczen",
        "Stycznia",
        "Styczniu",
        "Sty"
    ],
    "february": [
        "lut",
        "lutego",
        "luty",
        "Luty",
        "Lutego",
        "Lutym",
        "Lut"
    ],
    "march": [
        "mar",
        "marca",
        "marzec",
        "Marzec",
        "Marca",
        "Marcu",
        "Mar"
    ],
    "april": [
        "kwi",
        "kwiecień",
        "kwietnia",
        "Kwiecień",
        "Kwiecien",
        "Kwietnia",
        "Kwietniu",
        "Kwi",
        "Kwie"
    ],
    "may": [
        "maj",
        "maja",
        "Maj",
        "Maja",
        "Maju"
    ],
    "june": [
        "cze",
        "czerwca",
        "czerwiec",
        "Czerwiec",
        "Czerwca",
        "Czerwcu",
        "Cze"
    ],
    "july": [
        "lip",
        "lipca",
        "lipiec",
        "Lipiec",
        "Lipca",
        "Lipcu",
        "Lip"
    ],
    "august": [
        "sie",
        "sierpień",
        "sierpnia",
        "Sierpień",
        "Sierpien",
        "Sierpnia",
        "Sierpniu",
        "Sie"
    ],
    "september": [
        "wrz",
        "wrzesień",
        "września",
        "Wrzesień",
        "Wrzesien",
        "Września",
        "Wrzesnia",
        "Wrześniu",
        "Wrzesniu",
        "Wrz"
    ],
    "october": [
        "paź",
        "październik",
        "października",
        "Październik",
        "Pazdziernik",
        "Października",
        "Pazdziernika",
        "Październiku",
        "Pazdzierniku",
        "Paź",
        "Paz"
    ],
    "november": [
        "lis",
        "listopad",
        "listopada",
        "Listopad",
        "Listopada",
        "Listopadzie",
        "Lis"
    ],
    "december": [
        "gru",
        "grudnia",
        "grudzień",
        "Grudzień",
        "Grudzien",
        "Grudnia",
        "Grudniu",
        "Gru"
    ],
    "monday": [
        "pon",
        "poniedziałek",
        "Poniedziałek",
        "Poniedzialek",
        "Pon",
        "Pn"
    ],
    "tuesday": [
        "wt",
        "wtorek",
        "Wtorek",
        "Wt",
        "Wto"
    ],
    "wednesday": [
        "śr",
        "środa",
        "Środa",
        "Sroda",
        "Środę",
        "Srode",
        "Śro",
        "Sro",
        "Śr",
        "Sr"
    ],
    "thursday": [
        "czw",
        "czwartek",
        "Czwartek",
        "Czw",
        "Cz"
    ],
    "friday": [
        "piątek",
        "pt",
        "Piątek",
        "Piatek",
        "Pią",
        "Pia",
        "Pi",
        "Pt"
    ],
    "saturday": [
        "sob",
        "sobota",
        "Sobota",
        "Sobotę",
        "Sobote",
        "Sob",
        "So",
        "Sb"
    ],
    "sunday": [
        "niedz",
        "niedziela",
        "Niedziela",
        "Niedzielę",
        "Niedziele",
        "Nie",
        "Nd"
    ],
    "am": [
        "am"
    ],
    "pm": [
        "pm"
    ],
    "year": [
        "r",
        "rok",
        "lata",
        "lat"
    ],
    "month": [
        "mc",
        "mies",
        "miesiąc",
        "miesiac",
        "miesięcy",
        "miesiąca",
        "miesiące",
        "miesiace"
    ],
    "week": [
        "tydz",
        "tydzień",
        "tydzien",
        "tygodni",
        "tygodnie"
    ],
    "day": [
        "dzień",
        "dzien",
        "dnia",
        "dniu",
        "dni"
    ],
    "hour": [
        "g",
        "godz",
        "godzina",
        "godzinę",
        "godzine",
        "godzin",
        "godziny"
    ],
    "minute": [
        "min",
        "minuta",
        "minuty",
        "minut",
        "minute",
        "minutę"
    ],
    "second": [
        "s",
        "sek",
        "sekunda",
        "sekundę",
        "sekunde",
        "sekundy",
        "sekund"
    ],
    "relative-type": {
        "0 day ago": [
            "dzisiaj"
        ],
        "0 hour ago": [
            "ta godzina"
        ],
        "0 minute ago": [
            "ta minuta"
        ],
        "0 month ago": [
            "w tym miesiącu"
        ],
        "0 second ago": [
            "teraz"
        ],
        "0 week ago": [
            "w tym tygodniu"
        ],
        "0 year ago": [
            "w tym roku"
        ],
        "1 day ago": [
            "wczoraj"
        ],
        "1 month ago": [
            "w zeszłym miesiącu"
        ],
        "1 week ago": [
            "w zeszłym tygodniu"
        ],
        "1 year ago": [
            "w zeszłym roku"
        ],
        "in 1 day": [
            "jutro"
        ],
        "in 1 month": [
            "w przyszłym miesiącu"
        ],
        "in 1 week": [
            "w przyszłym tygodniu"
        ],
        "in 1 year": [
            "w przyszłym roku"
        ]
    },
    "relative-type-regex": {
        "\\1 day ago": [
            "(\\d+[.,]?\\d*) dnia temu",
            "(\\d+[.,]?\\d*) dzień temu"
        ],
        "\\1 hour ago": [
            "(\\d+[.,]?\\d*) g temu",
            "(\\d+[.,]?\\d*) godz temu",
            "(\\d+[.,]?\\d*) godziny temu",
            "(\\d+[.,]?\\d*) godzinę temu"
        ],
        "\\1 minute ago": [
            "(\\d+[.,]?\\d*) min temu",
            "(\\d+[.,]?\\d*) minuty temu",
            "(\\d+[.,]?\\d*) minutę temu"
        ],
        "\\1 month ago": [
            "(\\d+[.,]?\\d*) mies temu",
            "(\\d+[.,]?\\d*) miesiąc temu",
            "(\\d+[.,]?\\d*) miesiąca temu",
            "–(\\d+[.,]?\\d*) mies"
        ],
        "\\1 second ago": [
            "(\\d+[.,]?\\d*) s temu",
            "(\\d+[.,]?\\d*) sek temu",
            "(\\d+[.,]?\\d*) sekundy temu",
            "(\\d+[.,]?\\d*) sekundę temu"
        ],
        "\\1 week ago": [
            "(\\d+[.,]?\\d*) tydz temu",
            "(\\d+[.,]?\\d*) tydzień temu",
            "(\\d+[.,]?\\d*) tyg temu",
            "(\\d+[.,]?\\d*) tygodnia temu"
        ],
        "\\1 year ago": [
            "(\\d+[.,]?\\d*) rok temu",
            "(\\d+[.,]?\\d*) roku temu"
        ],
        "in \\1 day": [
            "za (\\d+[.,]?\\d*) dnia",
            "za (\\d+[.,]?\\d*) dzień"
        ],
        "in \\1 hour": [
            "za (\\d+[.,]?\\d*) g",
            "za (\\d+[.,]?\\d*) godz",
            "za (\\d+[.,]?\\d*) godziny",
            "za (\\d+[.,]?\\d*) godzinę"
        ],
        "in \\1 minute": [
            "za (\\d+[.,]?\\d*) min",
            "za (\\d+[.,]?\\d*) minuty",
            "za (\\d+[.,]?\\d*) minutę"
        ],
        "in \\1 month": [
            "za (\\d+[.,]?\\d*) mies",
            "za (\\d+[.,]?\\d*) miesiąc",
            "za (\\d+[.,]?\\d*) miesiąca"
        ],
        "in \\1 second": [
            "za (\\d+[.,]?\\d*) s",
            "za (\\d+[.,]?\\d*) sek",
            "za (\\d+[.,]?\\d*) sekundy",
            "za (\\d+[.,]?\\d*) sekundę"
        ],
        "in \\1 week": [
            "za (\\d+[.,]?\\d*) tydz",
            "za (\\d+[.,]?\\d*) tydzień",
            "za (\\d+[.,]?\\d*) tyg",
            "za (\\d+[.,]?\\d*) tygodnia"
        ],
        "in \\1 year": [
            "za (\\d+[.,]?\\d*) rok",
            "za (\\d+[.,]?\\d*) roku"
        ]
    },
    "locale_specific": {},
    "sentence_splitter_group": 1,
    "skip": [
        "i",
        "o",
        "r.",
        "roku",
        "w",
        " ",
        "'",
        ",",
        "-",
        ".",
        "/",
        ";",
        "@",
        "[",
        "]",
        "|",
        "，"
    ],
    "ago": [
        "temu"
    ],
    "in": [
        "za"
    ],
    "simplifications": [
        {
            "dziś": "0 dnia"
        },
        {
            "dzisiaj": "0 dnia"
        },
        {
            "onegdaj": "2 dnia"
        },
        {
            "przedwczoraj": "2 dnia temu"
        },
        {
            "jutro": "za 1 dnia"
        },
        {
            "pojutrze": "za 2 dnia"
        }
    ]
}



info = {
    "name": "yi",
    "date_order": "DMY",
    "january": [
        "יאַנ",
        "יאַנואַר"
    ],
    "february": [
        "פֿעב",
        "פֿעברואַר"
    ],
    "march": [
        "מערץ"
    ],
    "april": [
        "אַפּר",
        "אַפּריל"
    ],
    "may": [
        "מיי"
    ],
    "june": [
        "יוני"
    ],
    "july": [
        "יולי"
    ],
    "august": [
        "אויג",
        "אויגוסט"
    ],
    "september": [
        "סעפּ",
        "סעפּטעמבער"
    ],
    "october": [
        "אקט",
        "אקטאבער"
    ],
    "november": [
        "נאוו",
        "נאוועמבער"
    ],
    "december": [
        "דעצ",
        "דעצעמבער"
    ],
    "monday": [
        "מאָנטיק"
    ],
    "tuesday": [
        "דינסטיק"
    ],
    "wednesday": [
        "מיטוואך"
    ],
    "thursday": [
        "דאנערשטיק"
    ],
    "friday": [
        "פֿרײַטיק"
    ],
    "saturday": [
        "שבת"
    ],
    "sunday": [
        "זונטיק"
    ],
    "am": [
        "פֿאַרמיטאָג"
    ],
    "pm": [
        "נאָכמיטאָג"
    ],
    "year": [
        "יאָר"
    ],
    "month": [
        "מאנאַט"
    ],
    "week": [
        "וואך"
    ],
    "day": [
        "טאָג"
    ],
    "hour": [
        "שעה"
    ],
    "minute": [
        "מינוט"
    ],
    "second": [
        "סעקונדע"
    ],
    "relative-type": {
        "0 day ago": [
            "היינט"
        ],
        "0 hour ago": [
            "this hour"
        ],
        "0 minute ago": [
            "this minute"
        ],
        "0 month ago": [
            "דעם חודש"
        ],
        "0 second ago": [
            "now"
        ],
        "0 week ago": [
            "this week"
        ],
        "0 year ago": [
            "הײַ יאָר"
        ],
        "1 day ago": [
            "נעכטן"
        ],
        "1 month ago": [
            "פֿאַרגאנגענעם חודש"
        ],
        "1 week ago": [
            "last week"
        ],
        "1 year ago": [
            "פֿאַראַיאָר"
        ],
        "in 1 day": [
            "מארגן"
        ],
        "in 1 month": [
            "קומענדיקן חודש"
        ],
        "in 1 week": [
            "איבער אַכט טאָג"
        ],
        "in 1 year": [
            "איבער א יאָר"
        ]
    },
    "relative-type-regex": {
        "\\1 month ago": [
            "פֿאַר (\\d+[.,]?\\d*) חדשים",
            "פֿאַר (\\d+[.,]?\\d*) חודש"
        ],
        "\\1 year ago": [
            "פֿאַר (\\d+[.,]?\\d*) יאָר"
        ],
        "in \\1 day": [
            "אין (\\d+[.,]?\\d*) טאָג אַרום",
            "אין (\\d+[.,]?\\d*) טעג אַרום"
        ],
        "in \\1 month": [
            "איבער (\\d+[.,]?\\d*) חדשים",
            "איבער (\\d+[.,]?\\d*) חודש"
        ],
        "in \\1 year": [
            "איבער (\\d+[.,]?\\d*) יאָר"
        ]
    },
    "locale_specific": {},
    "skip": [
        " ",
        "'",
        ",",
        "-",
        ".",
        "/",
        ";",
        "@",
        "[",
        "]",
        "|",
        "，"
    ]
}



info = {
    "name": "he",
    "date_order": "DMY",
    "january": [
        "ינואר",
        "ינו׳",
        "לינואר",
        "בינואר",
        "ינו"
    ],
    "february": [
        "פברואר",
        "פבר׳",
        "בפברואר",
        "לפברואר",
        "פבר"
    ],
    "march": [
        "מרץ",
        "במרץ",
        "למרץ",
        "מרס",
        "במרס",
        "למרס",
        "מארס",
        "במארס",
        "למארס"
    ],
    "april": [
        "אפריל",
        "אפר׳",
        "באפריל",
        "לאפריל",
        "אפר"
    ],
    "may": [
        "מאי",
        "במאי",
        "למאי"
    ],
    "june": [
        "יוני",
        "ביוני",
        "ליוני",
        "יונ"
    ],
    "july": [
        "יולי",
        "ביולי",
        "ליולי",
        "יול"
    ],
    "august": [
        "אוגוסט",
        "אוג׳",
        "באוגוסט",
        "לאוגוסט",
        "אוג"
    ],
    "september": [
        "ספטמבר",
        "ספט׳",
        "בספטמבר",
        "לספטמבר",
        "ספט"
    ],
    "october": [
        "אוקטובר",
        "אוק׳",
        "באוקטובר",
        "לאוקטובר",
        "אוק"
    ],
    "november": [
        "נובמבר",
        "נוב׳",
        "בנובמבר",
        "לנובמבר",
        "נוב"
    ],
    "december": [
        "דצמבר",
        "דצמ׳",
        "בדצמבר",
        "לדצמבר",
        "דצמ"
    ],
    "monday": [
        "יום ב׳",
        "יום שני",
        "שני",
        "יום ב"
    ],
    "tuesday": [
        "יום ג׳",
        "יום שלישי",
        "שלישי",
        "יום ג"
    ],
    "wednesday": [
        "יום ד׳",
        "יום רביעי",
        "רביעי",
        "יום ד"
    ],
    "thursday": [
        "יום ה׳",
        "יום חמישי",
        "חמישי",
        "יום ה"
    ],
    "friday": [
        "יום ו׳",
        "יום שישי",
        "שישי",
        "יום ו"
    ],
    "saturday": [
        "יום שבת",
        "שבת"
    ],
    "sunday": [
        "יום א׳",
        "יום ראשון",
        "ראשון",
        "יום א"
    ],
    "am": [
        "לפנה״צ"
    ],
    "pm": [
        "אחה״צ"
    ],
    "year": [
        "שנה",
        "שנ׳",
        "שנים",
        "בשנה"
    ],
    "month": [
        "חודש",
        "חו׳",
        "חודשים",
        "בחודש"
    ],
    "week": [
        "שבוע",
        "שב׳",
        "שבועות"
    ],
    "day": [
        "יום",
        "ימים",
        "ביום"
    ],
    "hour": [
        "שעה",
        "שע׳",
        "שעות"
    ],
    "minute": [
        "דקה",
        "דק׳",
        "דקות"
    ],
    "second": [
        "שנייה",
        "שנ׳",
        "שניה",
        "שניות"
    ],
    "relative-type": {
        "0 day ago": [
            "היום"
        ],
        "0 hour ago": [
            "בשעה זו"
        ],
        "0 minute ago": [
            "בדקה זו"
        ],
        "0 month ago": [
            "החודש"
        ],
        "0 second ago": [
            "עכשיו"
        ],
        "0 week ago": [
            "השבוע"
        ],
        "0 year ago": [
            "השנה"
        ],
        "1 day ago": [
            "אתמול"
        ],
        "1 month ago": [
            "החודש שעבר"
        ],
        "1 week ago": [
            "השבוע שעבר"
        ],
        "1 year ago": [
            "השנה שעברה"
        ],
        "in 1 day": [
            "מחר"
        ],
        "in 1 month": [
            "החודש הבא"
        ],
        "in 1 week": [
            "השבוע הבא"
        ],
        "in 1 year": [
            "השנה הבאה"
        ]
    },
    "relative-type-regex": {
        "\\1 day ago": [
            "לפני (\\d+[.,]?\\d*) ימים",
            "לפני (\\d+[.,]?\\d*) ימ׳",
            "לפני יום (\\d+[.,]?\\d*)"
        ],
        "\\1 hour ago": [
            "לפני (\\d+[.,]?\\d*) שעות",
            "לפני (\\d+[.,]?\\d*) שע׳"
        ],
        "\\1 minute ago": [
            "לפני (\\d+[.,]?\\d*) דקות",
            "לפני (\\d+[.,]?\\d*) דק׳"
        ],
        "\\1 month ago": [
            "לפני (\\d+[.,]?\\d*) חודשים",
            "לפני (\\d+[.,]?\\d*) חו׳"
        ],
        "\\1 second ago": [
            "לפני (\\d+[.,]?\\d*) שניות",
            "לפני (\\d+[.,]?\\d*) שנ׳"
        ],
        "\\1 week ago": [
            "לפני (\\d+[.,]?\\d*) שבועות",
            "לפני (\\d+[.,]?\\d*) שב׳"
        ],
        "\\1 year ago": [
            "לפני (\\d+[.,]?\\d*) שנים"
        ],
        "in \\1 day": [
            "בעוד (\\d+[.,]?\\d*) ימים",
            "בעוד (\\d+[.,]?\\d*) ימ׳",
            "בעוד יום (\\d+[.,]?\\d*)"
        ],
        "in \\1 hour": [
            "בעוד (\\d+[.,]?\\d*) שעות",
            "בעוד (\\d+[.,]?\\d*) שע׳"
        ],
        "in \\1 minute": [
            "בעוד (\\d+[.,]?\\d*) דקות",
            "בעוד (\\d+[.,]?\\d*) דק׳"
        ],
        "in \\1 month": [
            "בעוד (\\d+[.,]?\\d*) חודשים",
            "בעוד (\\d+[.,]?\\d*) חו׳"
        ],
        "in \\1 second": [
            "בעוד (\\d+[.,]?\\d*) שניות",
            "בעוד (\\d+[.,]?\\d*) שנ׳"
        ],
        "in \\1 week": [
            "בעוד (\\d+[.,]?\\d*) שבועות",
            "בעוד (\\d+[.,]?\\d*) שב׳"
        ],
        "in \\1 year": [
            "בעוד (\\d+[.,]?\\d*) שנים"
        ]
    },
    "locale_specific": {},
    "sentence_splitter_group": 1,
    "skip": [
        "ב-",
        "בסביבות",
        "בערך",
        "בקירוב",
        "בשעה",
        "ה-",
        " ",
        "'",
        ",",
        "-",
        ".",
        "/",
        ";",
        "@",
        "[",
        "]",
        "|",
        "，"
    ],
    "ago": [
        "לפני"
    ],
    "in": [
        "בעוד",
        "עוד"
    ],
    "simplifications": [
        {
            "ויום": "1 יום"
        },
        {
            "ושבוע": "1 שבוע"
        },
        {
            "וחודש": "1 חודש"
        },
        {
            "ושנה": "1 שנה"
        },
        {
            "ו(\\w+)": "\\1"
        },
        {
            "שנתיי?ם": "2 שנים"
        },
        {
            "חודשיים": "2 חודשים"
        },
        {
            "שבועיי?ם": "2 שבועות"
        },
        {
            "יומיי?ם": "2 ימים"
        },
        {
            "שעתיי?ם": "2 שעות"
        },
        {
            "שלשום": "2 ימים"
        },
        {
            "מחר": "בעוד 1 יום"
        },
        {
            "אחר חצות": "am"
        },
        {
            "חצות": "12 am"
        },
        {
            "לפנות בוקר": "am"
        },
        {
            "בבוקר": "am"
        },
        {
            "בצהריי?ם": "pm"
        },
        {
            "אחרי ה?צהריי?ם": "pm"
        },
        {
            "אחה\"צ": "pm"
        },
        {
            "לפנות ערב": "pm"
        },
        {
            "בערב": "pm"
        }
    ]
}



from dateparser.data.languages_info import language_map


def map_languages(language_codes):
    """
    Returns the candidates from the supported languages codes.
    :param language_codes:
        A list of language codes, e.g. ['en', 'es'] in ISO 639 Standard.
    :type language_codes: list
    :return: Returns list[str] representing supported languages
    :rtype: list[str]
    """
    return [
        language_code
        for language in language_codes
        if language in language_map
        for language_code in language_map[language]
    ]



info = {
    "name": "ka",
    "date_order": "DMY",
    "january": [
        "იან",
        "იანვარი"
    ],
    "february": [
        "თებ",
        "თებერვალი"
    ],
    "march": [
        "მარ",
        "მარტი"
    ],
    "april": [
        "აპრ",
        "აპრილი"
    ],
    "may": [
        "მაი",
        "მაისი"
    ],
    "june": [
        "ივნ",
        "ივნისი"
    ],
    "july": [
        "ივლ",
        "ივლისი"
    ],
    "august": [
        "აგვ",
        "აგვისტო"
    ],
    "september": [
        "სექ",
        "სექტემბერი"
    ],
    "october": [
        "ოქტ",
        "ოქტომბერი"
    ],
    "november": [
        "ნოე",
        "ნოემბერი"
    ],
    "december": [
        "დეკ",
        "დეკემბერი"
    ],
    "monday": [
        "ორშ",
        "ორშაბათი"
    ],
    "tuesday": [
        "სამ",
        "სამშაბათი"
    ],
    "wednesday": [
        "ოთხ",
        "ოთხშაბათი"
    ],
    "thursday": [
        "ხუთ",
        "ხუთშაბათი"
    ],
    "friday": [
        "პარ",
        "პარასკევი"
    ],
    "saturday": [
        "შაბ",
        "შაბათი"
    ],
    "sunday": [
        "კვი",
        "კვირა"
    ],
    "am": [
        "am"
    ],
    "pm": [
        "pm",
        "შუადღ შემდეგ"
    ],
    "year": [
        "წ",
        "წელი",
        "წლის"
    ],
    "month": [
        "თვე"
    ],
    "week": [
        "კვ",
        "კვირა"
    ],
    "day": [
        "დღე"
    ],
    "hour": [
        "საათი",
        "სთ"
    ],
    "minute": [
        "წთ",
        "წუთი"
    ],
    "second": [
        "წამი",
        "წმ"
    ],
    "relative-type": {
        "0 day ago": [
            "დღეს"
        ],
        "0 hour ago": [
            "ამ საათში"
        ],
        "0 minute ago": [
            "ამ წუთში"
        ],
        "0 month ago": [
            "ამ თვეში"
        ],
        "0 second ago": [
            "ახლა"
        ],
        "0 week ago": [
            "ამ კვირაში"
        ],
        "0 year ago": [
            "ამ წელს"
        ],
        "1 day ago": [
            "გუშინ"
        ],
        "1 month ago": [
            "გასულ თვეს"
        ],
        "1 week ago": [
            "გასულ კვირაში"
        ],
        "1 year ago": [
            "გასულ წელს"
        ],
        "in 1 day": [
            "ხვალ"
        ],
        "in 1 month": [
            "მომავალ თვეს"
        ],
        "in 1 week": [
            "მომავალ კვირაში"
        ],
        "in 1 year": [
            "მომავალ წელს"
        ]
    },
    "relative-type-regex": {
        "\\1 day ago": [
            "(\\d+[.,]?\\d*) დღის წინ"
        ],
        "\\1 hour ago": [
            "(\\d+[.,]?\\d*) საათის წინ",
            "(\\d+[.,]?\\d*) სთ წინ"
        ],
        "\\1 minute ago": [
            "(\\d+[.,]?\\d*) წთ წინ",
            "(\\d+[.,]?\\d*) წუთის წინ"
        ],
        "\\1 month ago": [
            "(\\d+[.,]?\\d*) თვის წინ"
        ],
        "\\1 second ago": [
            "(\\d+[.,]?\\d*) წამის წინ",
            "(\\d+[.,]?\\d*) წმ წინ"
        ],
        "\\1 week ago": [
            "(\\d+[.,]?\\d*) კვ წინ",
            "(\\d+[.,]?\\d*) კვირის წინ"
        ],
        "\\1 year ago": [
            "(\\d+[.,]?\\d*) წლის წინ"
        ],
        "in \\1 day": [
            "(\\d+[.,]?\\d*) დღეში"
        ],
        "in \\1 hour": [
            "(\\d+[.,]?\\d*) საათში"
        ],
        "in \\1 minute": [
            "(\\d+[.,]?\\d*) წუთში"
        ],
        "in \\1 month": [
            "(\\d+[.,]?\\d*) თვეში"
        ],
        "in \\1 second": [
            "(\\d+[.,]?\\d*) წამში"
        ],
        "in \\1 week": [
            "(\\d+[.,]?\\d*) კვირაში"
        ],
        "in \\1 year": [
            "(\\d+[.,]?\\d*) წელიწადში",
            "(\\d+[.,]?\\d*) წელში"
        ]
    },
    "locale_specific": {},
    "skip": [
        "და",
        "დაახლოებით",
        "ზე",
        "ის",
        " ",
        "'",
        ",",
        "-",
        ".",
        "/",
        ";",
        "@",
        "[",
        "]",
        "|",
        "，"
    ],
    "sentence_splitter_group": 1,
    "ago": [
        "წინ"
    ],
    "in": [
        "დღეიდან"
    ],
    "simplifications": [
        {
            "ერთ": "1"
        }
    ]
}



info = {
    "name": "rw",
    "date_order": "YMD",
    "january": [
        "mut",
        "mutarama"
    ],
    "february": [
        "gas",
        "gashyantare"
    ],
    "march": [
        "wer",
        "werurwe"
    ],
    "april": [
        "mat",
        "mata"
    ],
    "may": [
        "gic",
        "gicuransi"
    ],
    "june": [
        "kam",
        "kamena"
    ],
    "july": [
        "nya",
        "nyakanga"
    ],
    "august": [
        "kan",
        "kanama"
    ],
    "september": [
        "nze",
        "nzeli"
    ],
    "october": [
        "ukw",
        "ukwakira"
    ],
    "november": [
        "ugu",
        "ugushyingo"
    ],
    "december": [
        "uku",
        "ukuboza"
    ],
    "monday": [
        "kuwa mbere",
        "mbe"
    ],
    "tuesday": [
        "kab",
        "kuwa kabiri"
    ],
    "wednesday": [
        "gtu",
        "kuwa gatatu"
    ],
    "thursday": [
        "kan",
        "kuwa kane"
    ],
    "friday": [
        "gnu",
        "kuwa gatanu"
    ],
    "saturday": [
        "gnd",
        "kuwa gatandatu"
    ],
    "sunday": [
        "cyu",
        "ku cyumweru"
    ],
    "am": [
        "am"
    ],
    "pm": [
        "pm"
    ],
    "year": [
        "year"
    ],
    "month": [
        "month"
    ],
    "week": [
        "week"
    ],
    "day": [
        "day"
    ],
    "hour": [
        "hour"
    ],
    "minute": [
        "minute"
    ],
    "second": [
        "second"
    ],
    "relative-type": {
        "0 day ago": [
            "today"
        ],
        "0 hour ago": [
            "this hour"
        ],
        "0 minute ago": [
            "this minute"
        ],
        "0 month ago": [
            "this month"
        ],
        "0 second ago": [
            "now"
        ],
        "0 week ago": [
            "this week"
        ],
        "0 year ago": [
            "this year"
        ],
        "1 day ago": [
            "yesterday"
        ],
        "1 month ago": [
            "last month"
        ],
        "1 week ago": [
            "last week"
        ],
        "1 year ago": [
            "last year"
        ],
        "in 1 day": [
            "tomorrow"
        ],
        "in 1 month": [
            "next month"
        ],
        "in 1 week": [
            "next week"
        ],
        "in 1 year": [
            "next year"
        ]
    },
    "locale_specific": {},
    "skip": [
        " ",
        "'",
        ",",
        "-",
        ".",
        "/",
        ";",
        "@",
        "[",
        "]",
        "|",
        "，"
    ]
}



info = {
    "name": "az",
    "date_order": "DMY",
    "january": [
        "yan",
        "yanvar"
    ],
    "february": [
        "fev",
        "fevral"
    ],
    "march": [
        "mar",
        "mart"
    ],
    "april": [
        "apr",
        "aprel"
    ],
    "may": [
        "may"
    ],
    "june": [
        "iyn",
        "iyun",
        "i̇yun"
    ],
    "july": [
        "iyl",
        "iyul",
        "i̇yul"
    ],
    "august": [
        "avq",
        "avqust"
    ],
    "september": [
        "sen",
        "sentyabr"
    ],
    "october": [
        "okt",
        "oktyabr"
    ],
    "november": [
        "noy",
        "noyabr"
    ],
    "december": [
        "dek",
        "dekabr"
    ],
    "monday": [
        "bazar ertəsi",
        "be"
    ],
    "tuesday": [
        "ça",
        "çərşənbə axşamı"
    ],
    "wednesday": [
        "ç",
        "çərşənbə"
    ],
    "thursday": [
        "ca",
        "cümə axşamı"
    ],
    "friday": [
        "c",
        "cümə"
    ],
    "saturday": [
        "ş",
        "şənbə"
    ],
    "sunday": [
        "b",
        "bazar"
    ],
    "am": [
        "am"
    ],
    "pm": [
        "pm"
    ],
    "year": [
        "il",
        "i̇l"
    ],
    "month": [
        "ay"
    ],
    "week": [
        "həftə"
    ],
    "day": [
        "gün"
    ],
    "hour": [
        "saat"
    ],
    "minute": [
        "dəq",
        "dəqiqə"
    ],
    "second": [
        "san",
        "saniyə"
    ],
    "relative-type": {
        "0 day ago": [
            "bu gün"
        ],
        "0 hour ago": [
            "bu saat"
        ],
        "0 minute ago": [
            "bu dəqiqə"
        ],
        "0 month ago": [
            "bu ay"
        ],
        "0 second ago": [
            "indi"
        ],
        "0 week ago": [
            "bu həftə"
        ],
        "0 year ago": [
            "bu il"
        ],
        "1 day ago": [
            "dünən"
        ],
        "1 month ago": [
            "keçən ay"
        ],
        "1 week ago": [
            "keçən həftə"
        ],
        "1 year ago": [
            "keçən il"
        ],
        "in 1 day": [
            "sabah"
        ],
        "in 1 month": [
            "gələn ay"
        ],
        "in 1 week": [
            "gələn həftə"
        ],
        "in 1 year": [
            "gələn il"
        ]
    },
    "relative-type-regex": {
        "\\1 day ago": [
            "(\\d+[.,]?\\d*) gün öncə"
        ],
        "\\1 hour ago": [
            "(\\d+[.,]?\\d*) saat öncə"
        ],
        "\\1 minute ago": [
            "(\\d+[.,]?\\d*) dəqiqə öncə"
        ],
        "\\1 month ago": [
            "(\\d+[.,]?\\d*) ay öncə"
        ],
        "\\1 second ago": [
            "(\\d+[.,]?\\d*) saniyə öncə"
        ],
        "\\1 week ago": [
            "(\\d+[.,]?\\d*) həftə öncə"
        ],
        "\\1 year ago": [
            "(\\d+[.,]?\\d*) il öncə"
        ],
        "in \\1 day": [
            "(\\d+[.,]?\\d*) gün ərzində"
        ],
        "in \\1 hour": [
            "(\\d+[.,]?\\d*) saat ərzində"
        ],
        "in \\1 minute": [
            "(\\d+[.,]?\\d*) dəqiqə ərzində"
        ],
        "in \\1 month": [
            "(\\d+[.,]?\\d*) ay ərzində"
        ],
        "in \\1 second": [
            "(\\d+[.,]?\\d*) saniyə ərzində"
        ],
        "in \\1 week": [
            "(\\d+[.,]?\\d*) həftə ərzində"
        ],
        "in \\1 year": [
            "(\\d+[.,]?\\d*) il ərzində"
        ]
    },
    "locale_specific": {},
    "skip": [
        " ",
        "'",
        ",",
        "-",
        ".",
        "/",
        ";",
        "@",
        "[",
        "]",
        "|",
        "，"
    ]
}



info = {
    "name": "lrc",
    "date_order": "YMD",
    "january": [
        "جانڤیە"
    ],
    "february": [
        "فئڤریە"
    ],
    "march": [
        "مارس"
    ],
    "april": [
        "آڤریل"
    ],
    "may": [
        "مئی"
    ],
    "june": [
        "جوٙأن"
    ],
    "july": [
        "جوٙلا"
    ],
    "august": [
        "آگوست"
    ],
    "september": [
        "سئپتامر"
    ],
    "october": [
        "ئوکتوڤر"
    ],
    "november": [
        "نوڤامر"
    ],
    "december": [
        "دئسامر"
    ],
    "monday": [
        "mon"
    ],
    "tuesday": [
        "tue"
    ],
    "wednesday": [
        "wed"
    ],
    "thursday": [
        "thu"
    ],
    "friday": [
        "fri"
    ],
    "saturday": [
        "sat"
    ],
    "sunday": [
        "sun"
    ],
    "am": [
        "am"
    ],
    "pm": [
        "pm"
    ],
    "year": [
        "سال"
    ],
    "month": [
        "ما"
    ],
    "week": [
        "ھأفتە"
    ],
    "day": [
        "روٙز"
    ],
    "hour": [
        "ساأت"
    ],
    "minute": [
        "دئیقە"
    ],
    "second": [
        "ثانیە"
    ],
    "relative-type": {
        "0 day ago": [
            "أمروٙ"
        ],
        "0 hour ago": [
            "this hour"
        ],
        "0 minute ago": [
            "this minute"
        ],
        "0 month ago": [
            "this month"
        ],
        "0 second ago": [
            "now"
        ],
        "0 week ago": [
            "this week"
        ],
        "0 year ago": [
            "this year"
        ],
        "1 day ago": [
            "دیروٙز"
        ],
        "1 month ago": [
            "last month"
        ],
        "1 week ago": [
            "last week"
        ],
        "1 year ago": [
            "last year"
        ],
        "in 1 day": [
            "شوٙصوٙ"
        ],
        "in 1 month": [
            "next month"
        ],
        "in 1 week": [
            "next week"
        ],
        "in 1 year": [
            "next year"
        ]
    },
    "locale_specific": {
        "lrc-IQ": {
            "name": "lrc-IQ"
        }
    },
    "skip": [
        " ",
        "'",
        ",",
        "-",
        ".",
        "/",
        ";",
        "@",
        "[",
        "]",
        "|",
        "，"
    ]
}



info = {
    "name": "ha",
    "date_order": "DMY",
    "january": [
        "jan",
        "janairu"
    ],
    "february": [
        "fab",
        "faburairu"
    ],
    "march": [
        "mar",
        "maris"
    ],
    "april": [
        "afi",
        "afirilu"
    ],
    "may": [
        "may",
        "mayu"
    ],
    "june": [
        "yun",
        "yuni"
    ],
    "july": [
        "yul",
        "yuli"
    ],
    "august": [
        "agu",
        "agusta"
    ],
    "september": [
        "sat",
        "satumba"
    ],
    "october": [
        "okt",
        "oktoba"
    ],
    "november": [
        "nuw",
        "nuwamba"
    ],
    "december": [
        "dis",
        "disamba"
    ],
    "monday": [
        "lit",
        "litinin"
    ],
    "tuesday": [
        "tal",
        "talata"
    ],
    "wednesday": [
        "lar",
        "laraba"
    ],
    "thursday": [
        "alh",
        "alhamis"
    ],
    "friday": [
        "jum",
        "jumma'a"
    ],
    "saturday": [
        "asa",
        "asabar"
    ],
    "sunday": [
        "lah",
        "lahadi"
    ],
    "am": [
        "am"
    ],
    "pm": [
        "pm"
    ],
    "year": [
        "shekara"
    ],
    "month": [
        "wata"
    ],
    "week": [
        "mako"
    ],
    "day": [
        "kwana"
    ],
    "hour": [
        "awa"
    ],
    "minute": [
        "minti"
    ],
    "second": [
        "daƙiƙa"
    ],
    "relative-type": {
        "0 day ago": [
            "yau"
        ],
        "0 hour ago": [
            "this hour"
        ],
        "0 minute ago": [
            "this minute"
        ],
        "0 month ago": [
            "this month"
        ],
        "0 second ago": [
            "now"
        ],
        "0 week ago": [
            "this week"
        ],
        "0 year ago": [
            "this year"
        ],
        "1 day ago": [
            "jiya"
        ],
        "1 month ago": [
            "last month"
        ],
        "1 week ago": [
            "last week"
        ],
        "1 year ago": [
            "last year"
        ],
        "in 1 day": [
            "gobe"
        ],
        "in 1 month": [
            "next month"
        ],
        "in 1 week": [
            "next week"
        ],
        "in 1 year": [
            "next year"
        ]
    },
    "locale_specific": {
        "ha-GH": {
            "name": "ha-GH"
        },
        "ha-NE": {
            "name": "ha-NE"
        }
    },
    "skip": [
        " ",
        "'",
        ",",
        "-",
        ".",
        "/",
        ";",
        "@",
        "[",
        "]",
        "|",
        "，"
    ]
}



info = {
    "name": "tr",
    "date_order": "DMY",
    "january": [
        "oca",
        "ocak",
        "Oc"
    ],
    "february": [
        "şub",
        "şubat",
        "Şu"
    ],
    "march": [
        "mar",
        "mart"
    ],
    "april": [
        "nis",
        "nisan",
        "Ni"
    ],
    "may": [
        "may",
        "mayıs"
    ],
    "june": [
        "haz",
        "haziran",
        "Ha"
    ],
    "july": [
        "tem",
        "temmuz",
        "Te"
    ],
    "august": [
        "ağu",
        "ağustos",
        "Ağ"
    ],
    "september": [
        "eyl",
        "eylül",
        "Ey"
    ],
    "october": [
        "eki",
        "ekim",
        "Ek"
    ],
    "november": [
        "kas",
        "kasım",
        "Ka"
    ],
    "december": [
        "ara",
        "aralık",
        "Ar"
    ],
    "monday": [
        "pazartesi",
        "pzt"
    ],
    "tuesday": [
        "sal",
        "salı"
    ],
    "wednesday": [
        "çar",
        "çarşamba",
        "Çrs"
    ],
    "thursday": [
        "per",
        "perşembe",
        "Prs"
    ],
    "friday": [
        "cum",
        "cuma"
    ],
    "saturday": [
        "cmt",
        "cumartesi"
    ],
    "sunday": [
        "paz",
        "pazar"
    ],
    "am": [
        "öö"
    ],
    "pm": [
        "ös"
    ],
    "year": [
        "yıl",
        "sene"
    ],
    "month": [
        "ay"
    ],
    "week": [
        "hafta",
        "hf"
    ],
    "day": [
        "gün"
    ],
    "hour": [
        "sa",
        "saat"
    ],
    "minute": [
        "dakika",
        "dk"
    ],
    "second": [
        "saniye",
        "sn"
    ],
    "relative-type": {
        "0 day ago": [
            "bugün"
        ],
        "0 hour ago": [
            "bu saat"
        ],
        "0 minute ago": [
            "bu dakika"
        ],
        "0 month ago": [
            "bu ay"
        ],
        "0 second ago": [
            "şimdi"
        ],
        "0 week ago": [
            "bu hafta"
        ],
        "0 year ago": [
            "bu yıl"
        ],
        "1 day ago": [
            "dün",
            "geçen gün"
        ],
        "1 month ago": [
            "geçen ay"
        ],
        "1 week ago": [
            "geçen hafta"
        ],
        "1 year ago": [
            "geçen yıl"
        ],
        "in 1 day": [
            "yarın",
            "önümüzdeki gün"
        ],
        "in 1 month": [
            "gelecek ay",
            "önümüzdeki ay"
        ],
        "in 1 week": [
            "gelecek hafta",
            "haftaya",
            "önümüzdeki hafta"
        ],
        "in 1 year": [
            "gelecek yıl",
            "önümüzdeki yıl"
        ]
    },
    "relative-type-regex": {
        "\\1 day ago": [
            "(\\d+[.,]?\\d*) gün önce"
        ],
        "\\1 hour ago": [
            "(\\d+[.,]?\\d*) sa önce",
            "(\\d+[.,]?\\d*) saat önce"
        ],
        "\\1 minute ago": [
            "(\\d+[.,]?\\d*) dakika önce",
            "(\\d+[.,]?\\d*) dk önce"
        ],
        "\\1 month ago": [
            "(\\d+[.,]?\\d*) ay önce"
        ],
        "\\1 second ago": [
            "(\\d+[.,]?\\d*) saniye önce",
            "(\\d+[.,]?\\d*) sn önce"
        ],
        "\\1 week ago": [
            "(\\d+[.,]?\\d*) hafta önce",
            "(\\d+[.,]?\\d*) hf önce"
        ],
        "\\1 year ago": [
            "(\\d+[.,]?\\d*) yıl önce"
        ],
        "in \\1 day": [
            "(\\d+[.,]?\\d*) gün sonra"
        ],
        "in \\1 hour": [
            "(\\d+[.,]?\\d*) sa sonra",
            "(\\d+[.,]?\\d*) saat sonra"
        ],
        "in \\1 minute": [
            "(\\d+[.,]?\\d*) dakika sonra",
            "(\\d+[.,]?\\d*) dk sonra"
        ],
        "in \\1 month": [
            "(\\d+[.,]?\\d*) ay sonra"
        ],
        "in \\1 second": [
            "(\\d+[.,]?\\d*) saniye sonra",
            "(\\d+[.,]?\\d*) sn sonra"
        ],
        "in \\1 week": [
            "(\\d+[.,]?\\d*) hafta sonra",
            "(\\d+[.,]?\\d*) hf sonra"
        ],
        "in \\1 year": [
            "(\\d+[.,]?\\d*) yıl sonra"
        ]
    },
    "locale_specific": {
        "tr-CY": {
            "name": "tr-CY"
        }
    },
    "skip": [
        "ve",
        "yaklaşık",
        " ",
        "'",
        ",",
        "-",
        ".",
        "/",
        ";",
        "@",
        "[",
        "]",
        "|",
        "，"
    ],
    "sentence_splitter_group": 1,
    "ago": [
        "önce"
    ],
    "in": [
        "içerisinde",
        "içinde",
        "sonra"
    ]
}



import collections
import sys
from collections.abc import Set
from datetime import datetime, timedelta

import regex as re
from tzlocal import get_localzone
from dateutil.relativedelta import relativedelta

from dateparser.date_parser import date_parser
from dateparser.freshness_date_parser import freshness_date_parser
from dateparser.languages.loader import LocaleDataLoader
from dateparser.conf import apply_settings, check_settings
from dateparser.parser import _parse_absolute, _parse_nospaces
from dateparser.timezone_parser import pop_tz_offset_from_string
from dateparser.utils import apply_timezone_from_settings, \
    set_correct_day_from_settings, \
    get_timezone_from_tz_string
from dateparser.custom_language_detection.language_mapping import map_languages

APOSTROPHE_LOOK_ALIKE_CHARS = [
    '\N{RIGHT SINGLE QUOTATION MARK}',     # '\u2019'
    '\N{MODIFIER LETTER APOSTROPHE}',      # '\u02bc'
    '\N{MODIFIER LETTER TURNED COMMA}',    # '\u02bb'
    '\N{ARMENIAN APOSTROPHE}',             # '\u055a'
    '\N{LATIN SMALL LETTER SALTILLO}',     # '\ua78c'
    '\N{PRIME}',                           # '\u2032'
    '\N{REVERSED PRIME}',                  # '\u2035'
    '\N{MODIFIER LETTER PRIME}',           # '\u02b9'
    '\N{FULLWIDTH APOSTROPHE}',            # '\uff07'
]

RE_NBSP = re.compile('\xa0', flags=re.UNICODE)
RE_SPACES = re.compile(r'\s+')
RE_TRIM_SPACES = re.compile(r'^\s+(\S.*?)\s+$')
RE_TRIM_COLONS = re.compile(r'(\S.*?):*$')

RE_SANITIZE_SKIP = re.compile(r'\t|\n|\r|\u00bb|,\s\u0432\b|\u200e|\xb7|\u200f|\u064e|\u064f', flags=re.M)
RE_SANITIZE_RUSSIAN = re.compile(r'([\W\d])\u0433\.', flags=re.I | re.U)
RE_SANITIZE_CROATIAN = re.compile(r'(\d+)\.\s?(\d+)\.\s?(\d+)\.( u)?', flags=re.I | re.U)
RE_SANITIZE_PERIOD = re.compile(r'(?<=[^0-9\s])\.', flags=re.U)
RE_SANITIZE_ON = re.compile(r'^.*?on:\s+(.*)')
RE_SANITIZE_APOSTROPHE = re.compile('|'.join(APOSTROPHE_LOOK_ALIKE_CHARS))

RE_SEARCH_TIMESTAMP = re.compile(r'^(\d{10})(\d{3})?(\d{3})?(?![^.])')
RE_SEARCH_NEGATIVE_TIMESTAMP = re.compile(r'^([-]\d{10})(\d{3})?(\d{3})?(?![^.])')


def sanitize_spaces(date_string):
    date_string = RE_NBSP.sub(' ', date_string)
    date_string = RE_SPACES.sub(' ', date_string)
    date_string = RE_TRIM_SPACES.sub(r'\1', date_string)
    return date_string


def date_range(begin, end, **kwargs):
    dateutil_error_prone_args = ['year', 'month', 'week', 'day', 'hour',
                                 'minute', 'second']
    for arg in dateutil_error_prone_args:
        if arg in kwargs:
            raise ValueError("Invalid argument: %s" % arg)

    step = relativedelta(**kwargs) if kwargs else relativedelta(days=1)

    date = begin
    while date < end:
        yield date
        date += step

    # handles edge-case when iterating months and last interval is < 30 days
    if kwargs.get('months', 0) > 0 and (date.year, date.month) == (end.year, end.month):
        yield end


def get_intersecting_periods(low, high, period='day'):
    if period not in ['year', 'month', 'week', 'day', 'hour', 'minute', 'second', 'microsecond']:
        raise ValueError("Invalid period: {}".format(period))

    if high <= low:
        return

    step = relativedelta(**{period + 's': 1})

    current_period_start = low
    if isinstance(current_period_start, datetime):
        reset_arguments = {}
        for test_period in ['microsecond', 'second', 'minute', 'hour']:
            if test_period == period:
                break
            else:
                reset_arguments[test_period] = 0
        current_period_start = current_period_start.replace(**reset_arguments)

    if period == 'week':
        current_period_start \
            = current_period_start - timedelta(days=current_period_start.weekday())
    elif period == 'month':
        current_period_start = current_period_start.replace(day=1)
    elif period == 'year':
        current_period_start = current_period_start.replace(month=1, day=1)

    while current_period_start < high:
        yield current_period_start
        current_period_start += step


def sanitize_date(date_string):
    date_string = RE_SANITIZE_SKIP.sub(' ', date_string)
    date_string = RE_SANITIZE_RUSSIAN.sub(r'\1 ', date_string)  # remove 'г.' (Russian for year) but not in words
    date_string = RE_SANITIZE_CROATIAN.sub(r'\1.\2.\3 ', date_string)  # extra '.' and 'u' interferes with parsing relative fractional dates
    date_string = sanitize_spaces(date_string)
    date_string = RE_SANITIZE_PERIOD.sub('', date_string)
    date_string = RE_SANITIZE_ON.sub(r'\1', date_string)
    date_string = RE_TRIM_COLONS.sub(r'\1', date_string)
    date_string = RE_SANITIZE_APOSTROPHE.sub("'", date_string)
    date_string = date_string.strip()
    return date_string


def get_date_from_timestamp(date_string, settings, negative=False):
    if negative:
        match = RE_SEARCH_NEGATIVE_TIMESTAMP.search(date_string)
    else:
        match = RE_SEARCH_TIMESTAMP.search(date_string)

    if match:
        if (settings is None or
            settings.TIMEZONE is None or
            'local' in settings.TIMEZONE.lower()):
            # If the timezone in settings is unset, or it's 'local', use the
            # local timezone
            timezone = get_localzone()
        else:
            # Otherwise, use the timezone given in settings
            timezone = get_timezone_from_tz_string(settings.TIMEZONE)

        seconds = int(match.group(1))
        millis = int(match.group(2) or 0)
        micros = int(match.group(3) or 0)
        date_obj = (datetime
                    .fromtimestamp(seconds, timezone)
                    .replace(microsecond=millis * 1000 + micros, tzinfo=None)
                    )
        date_obj = apply_timezone_from_settings(date_obj, settings)
        return date_obj


def parse_with_formats(date_string, date_formats, settings):
    """ Parse with formats and return a dictionary with 'period' and 'obj_date'.

    :returns: :class:`datetime.datetime`, dict or None

    """
    period = 'day'
    for date_format in date_formats:
        try:
            date_obj = datetime.strptime(date_string, date_format)
        except ValueError:
            continue
        else:
            if '%d' not in date_format:
                period = 'month'
                date_obj = set_correct_day_from_settings(date_obj, settings)

            if not ('%y' in date_format or '%Y' in date_format):
                today = datetime.today()
                date_obj = date_obj.replace(year=today.year)

            date_obj = apply_timezone_from_settings(date_obj, settings)

            return DateData(date_obj=date_obj, period=period)
    else:
        return DateData(date_obj=None, period=period)


class _DateLocaleParser:

    def __init__(self, locale, date_string, date_formats, settings=None):
        self._settings = settings
        if not (date_formats is None or isinstance(date_formats, (list, tuple, Set))):
            raise TypeError("Date formats should be list, tuple or set of strings")

        self.locale = locale
        self.date_string = date_string
        self.date_formats = date_formats
        self._translated_date = None
        self._translated_date_with_formatting = None
        self._parsers = {
            'timestamp': self._try_timestamp,
            'negative-timestamp': self._try_negative_timestamp,
            'relative-time': self._try_freshness_parser,
            'custom-formats': self._try_given_formats,
            'absolute-time': self._try_absolute_parser,
            'no-spaces-time': self._try_nospaces_parser,
        }

    @classmethod
    def parse(cls, locale, date_string, date_formats=None, settings=None):
        instance = cls(locale, date_string, date_formats, settings)
        return instance._parse()

    def _parse(self):
        for parser_name in self._settings.PARSERS:
            date_data = self._parsers[parser_name]()
            if self._is_valid_date_data(date_data):
                return date_data
        else:
            return None

    def _try_timestamp_parser(self, negative=False):
        return DateData(
            date_obj=get_date_from_timestamp(self.date_string, self._settings, negative=negative),
            period='time' if self._settings.RETURN_TIME_AS_PERIOD else 'day',
        )

    def _try_timestamp(self):
        return self._try_timestamp_parser()

    def _try_negative_timestamp(self):
        return self._try_timestamp_parser(negative=True)

    def _try_freshness_parser(self):
        try:
            return freshness_date_parser.get_date_data(self._get_translated_date(), self._settings)
        except (OverflowError, ValueError):
            return None

    def _try_absolute_parser(self):
        return self._try_parser(parse_method=_parse_absolute)

    def _try_nospaces_parser(self):
        return self._try_parser(parse_method=_parse_nospaces)

    def _try_parser(self, parse_method):
        _order = self._settings.DATE_ORDER
        try:
            if self._settings.PREFER_LOCALE_DATE_ORDER:
                if 'DATE_ORDER' not in self._settings._mod_settings:
                    self._settings.DATE_ORDER = self.locale.info.get('date_order', _order)
            date_obj, period = date_parser.parse(
                self._get_translated_date(), parse_method=parse_method, settings=self._settings)
            self._settings.DATE_ORDER = _order
            return DateData(
                date_obj=date_obj,
                period=period,
            )
        except ValueError:
            self._settings.DATE_ORDER = _order
            return None

    def _try_given_formats(self):
        if not self.date_formats:
            return

        return parse_with_formats(
            self._get_translated_date_with_formatting(),
            self.date_formats, settings=self._settings
        )

    def _get_translated_date(self):
        if self._translated_date is None:
            self._translated_date = self.locale.translate(
                self.date_string, keep_formatting=False, settings=self._settings)
        return self._translated_date

    def _get_translated_date_with_formatting(self):
        if self._translated_date_with_formatting is None:
            self._translated_date_with_formatting = self.locale.translate(
                self.date_string, keep_formatting=True, settings=self._settings)
        return self._translated_date_with_formatting

    def _is_valid_date_data(self, date_data):
        if not isinstance(date_data, DateData):
            return False
        if not date_data['date_obj'] or not date_data['period']:
            return False
        if date_data['date_obj'] and not isinstance(date_data['date_obj'], datetime):
            return False
        if date_data['period'] not in ('time', 'day', 'week', 'month', 'year'):
            return False
        return True


class DateData:
    """
    Class that represents the parsed data with useful information.
    It can be accessed with square brackets like a dict object.
    """

    def __init__(self, *, date_obj=None, period=None, locale=None):
        self.date_obj = date_obj
        self.period = period
        self.locale = locale

    def __getitem__(self, k):
        if not hasattr(self, k):
            raise KeyError(k)
        return getattr(self, k)

    def __setitem__(self, k, v):
        if not hasattr(self, k):
            raise KeyError(k)
        setattr(self, k, v)

    def __repr__(self):
        properties_text = ', '.join('{}={}'.format(prop, val.__repr__()) for prop, val in self.__dict__.items())

        return '{}({})'.format(
            self.__class__.__name__, properties_text
        )


class DateDataParser:
    """
    Class which handles language detection, translation and subsequent generic parsing of
    string representing date and/or time.

    :param languages:
        A list of language codes, e.g. ['en', 'es', 'zh-Hant'].
        If locales are not given, languages and region are
        used to construct locales for translation.
    :type languages: list

    :param locales:
        A list of locale codes, e.g. ['fr-PF', 'qu-EC', 'af-NA'].
        The parser uses only these locales to translate date string.
    :type locales: list

    :param region:
        A region code, e.g. 'IN', '001', 'NE'.
        If locales are not given, languages and region are
        used to construct locales for translation.
    :type region: str

    :param try_previous_locales:
        If True, locales previously used to translate date are tried first.
    :type try_previous_locales: bool

    :param use_given_order:
        If True, locales are tried for translation of date string
        in the order in which they are given.
    :type use_given_order: bool

    :param settings:
        Configure customized behavior using settings defined in :mod:`dateparser.conf.Settings`.
    :type settings: dict

    :param detect_languages_function:
        A function for language detection that takes as input a `text` and a `confidence_threshold`,
        and returns a list of detected language codes.
        Note: this function is only used if ``languages`` and ``locales`` are not provided.
    :type detect_languages_function: function

    :return: A parser instance

    :raises:
         ``ValueError``: Unknown Language, ``TypeError``: Languages argument must be a list,
         ``SettingValidationError``: A provided setting is not valid.
    """

    locale_loader = None

    @apply_settings
    def __init__(self, languages=None, locales=None, region=None, try_previous_locales=False,
                 use_given_order=False, settings=None, detect_languages_function=None):

        if languages is not None and not isinstance(languages, (list, tuple, Set)):
            raise TypeError("languages argument must be a list (%r given)" % type(languages))

        if locales is not None and not isinstance(locales, (list, tuple, Set)):
            raise TypeError("locales argument must be a list (%r given)" % type(locales))

        if region is not None and not isinstance(region, str):
            raise TypeError("region argument must be str (%r given)" % type(region))

        if not isinstance(try_previous_locales, bool):
            raise TypeError("try_previous_locales argument must be a boolean (%r given)"
                            % type(try_previous_locales))

        if not isinstance(use_given_order, bool):
            raise TypeError("use_given_order argument must be a boolean (%r given)"
                            % type(use_given_order))

        if not locales and not languages and use_given_order:
            raise ValueError("locales or languages must be given if use_given_order is True")

        check_settings(settings)

        self._settings = settings
        self.try_previous_locales = try_previous_locales
        self.use_given_order = use_given_order
        self.languages = list(languages) if languages else None
        self.locales = locales
        self.region = region
        self.detect_languages_function = detect_languages_function
        self.previous_locales = collections.OrderedDict()

    def get_date_data(self, date_string, date_formats=None):
        """
        Parse string representing date and/or time in recognizable localized formats.
        Supports parsing multiple languages and timezones.

        :param date_string:
            A string representing date and/or time in a recognizably valid format.
        :type date_string: str
        :param date_formats:
            A list of format strings using directives as given
            `here <https://docs.python.org/2/library/datetime.html#strftime-and-strptime-behavior>`_.
            The parser applies formats one by one, taking into account the detected languages.
        :type date_formats: list

        :return: a ``DateData`` object.

        :raises: ValueError - Unknown Language

        .. note:: *Period* values can be a 'day' (default), 'week', 'month', 'year', 'time'.

        *Period* represents the granularity of date parsed from the given string.

        In the example below, since no day information is present, the day is assumed to be current
        day ``16`` from *current date* (which is June 16, 2015, at the moment of writing this).
        Hence, the level of precision is ``month``:

            >>> DateDataParser().get_date_data('March 2015')
            DateData(date_obj=datetime.datetime(2015, 3, 16, 0, 0), period='month', locale='en')

        Similarly, for date strings with no day and month information present, level of precision
        is ``year`` and day ``16`` and month ``6`` are from *current_date*.

            >>> DateDataParser().get_date_data('2014')
            DateData(date_obj=datetime.datetime(2014, 6, 16, 0, 0), period='year', locale='en')

        Dates with time zone indications or UTC offsets are returned in UTC time unless
        specified using `Settings <https://dateparser.readthedocs.io/en/latest/settings.html#settings>`__.

            >>> DateDataParser().get_date_data('23 March 2000, 1:21 PM CET')
            DateData(date_obj=datetime.datetime(2000, 3, 23, 13, 21, tzinfo=<StaticTzInfo 'CET'>),
            period='day', locale='en')

        """
        if not isinstance(date_string, str):
            raise TypeError('Input type must be str')

        res = parse_with_formats(date_string, date_formats or [], self._settings)
        if res['date_obj']:
            return res

        date_string = sanitize_date(date_string)

        for locale in self._get_applicable_locales(date_string):
            parsed_date = _DateLocaleParser.parse(
                locale, date_string, date_formats, settings=self._settings)
            if parsed_date:
                parsed_date['locale'] = locale.shortname
                if self.try_previous_locales:
                    self.previous_locales[locale] = None
                return parsed_date
        else:
            return DateData(date_obj=None, period='day', locale=None)

    def get_date_tuple(self, *args, **kwargs):
        date_data = self.get_date_data(*args, **kwargs)
        fields = date_data.__dict__.keys()
        date_tuple = collections.namedtuple('DateData', fields)
        return date_tuple(**date_data.__dict__)

    def _get_applicable_locales(self, date_string):
        pop_tz_cache = []

        def date_strings():
            """ A generator instead of a static list to avoid calling
            pop_tz_offset_from_string if the first locale matches on unmodified
            date_string.
            """
            yield date_string
            if not pop_tz_cache:
                stripped_date_string, _ = pop_tz_offset_from_string(
                    date_string, as_offset=False)
                if stripped_date_string == date_string:
                    stripped_date_string = None
                pop_tz_cache[:] = [stripped_date_string]
            stripped_date_string, = pop_tz_cache
            if stripped_date_string is not None:
                yield stripped_date_string

        if self.try_previous_locales:
            for locale in self.previous_locales.keys():
                for s in date_strings():
                    if self._is_applicable_locale(locale, s):
                        yield locale

        if self.detect_languages_function and not self.languages and not self.locales:
            detected_languages = self.detect_languages_function(
                text=date_string, confidence_threshold=self._settings.LANGUAGE_DETECTION_CONFIDENCE_THRESHOLD
            )

            self.languages = map_languages(detected_languages)

        for locale in self._get_locale_loader().get_locales(
                languages=self.languages, locales=self.locales, region=self.region,
                use_given_order=self.use_given_order):
            for s in date_strings():
                if self._is_applicable_locale(locale, s):
                    yield locale

        if self._settings.DEFAULT_LANGUAGES:
            for locale in self._get_locale_loader().get_locales(
                languages=self._settings.DEFAULT_LANGUAGES, locales=None,
                region=self.region, use_given_order=self.use_given_order
            ):
                yield locale

    def _is_applicable_locale(self, locale, date_string):
        return locale.is_applicable(
            date_string,
            strip_timezone=False,  # it is stripped outside
            settings=self._settings)

    @classmethod
    def _get_locale_loader(cls):
        if not cls.locale_loader:
            cls.locale_loader = LocaleDataLoader()
        return cls.locale_loader



from dateparser.date import DateData
from dateparser.parser import _parser
from dateparser.conf import settings
from datetime import datetime


class CalendarBase:
    """Base setup class for non-Gregorian calendar system.

    :param source:
        Date string passed to calendar parser.
    :type source: str
    """

    parser = NotImplemented

    def __init__(self, source):
        self.source = source

    def get_date(self):
        try:
            date_obj, period = self.parser.parse(self.source, settings)
            return DateData(date_obj=date_obj, period=period)
        except ValueError:
            pass


class non_gregorian_parser(_parser):

    calendar_converter = NotImplemented
    default_year = NotImplemented
    default_month = NotImplemented
    default_day = NotImplemented
    non_gregorian_date_cls = NotImplemented

    _digits = None
    _months = None
    _weekdays = None
    _number_letters = None

    @classmethod
    def _replace_time_conventions(cls, source):
        return source

    @classmethod
    def _replace_digits(cls, source):
        return source

    @classmethod
    def _replace_months(cls, source):
        return source

    @classmethod
    def _replace_weekdays(cls, source):
        return source

    @classmethod
    def _replace_time(cls, source):
        return source

    @classmethod
    def _replace_days(cls, source):
        return source

    @classmethod
    def to_latin(cls, source):
        result = source
        result = cls._replace_months(result)
        result = cls._replace_weekdays(result)
        result = cls._replace_digits(result)
        result = cls._replace_days(result)
        result = cls._replace_time(result)
        result = cls._replace_time_conventions(result)

        result = result.strip()

        return result

    def _get_datetime_obj(self, **params):
        day = params['day']
        year = params['year']
        month = params['month']
        if (
            not(0 < day <= self.calendar_converter.month_length(year, month))
            and not(self._token_day or hasattr(self, '_token_weekday'))
        ):
            day = self.calendar_converter.month_length(year, month)
        year, month, day = self.calendar_converter.to_gregorian(year=year, month=month, day=day)
        c_params = params.copy()
        c_params.update(dict(year=year, month=month, day=day))
        return datetime(**c_params)

    def _get_datetime_obj_params(self):
        if not self.now:
            self._set_relative_base()
        now_year, now_month, now_day = self.calendar_converter.from_gregorian(
            self.now.year, self.now.month, self.now.day)
        params = {
            'day': self.day or now_day,
            'month': self.month or now_month,
            'year': self.year or now_year,
            'hour': 0, 'minute': 0, 'second': 0, 'microsecond': 0,
        }
        return params

    def _get_date_obj(self, token, directive):
        year, month, day = self.default_year, self.default_month, self.default_day
        token_len = len(token)
        is_digit = token.isdigit()
        if directive == '%A' and self._weekdays and token.title() in self._weekdays:
            pass
        elif (
            directive == '%m'
            and token_len <= 2
            and is_digit
            and 1 <= int(token) <= 12
        ):
            month = int(token)
        elif directive == '%B' and self._months and token in self._months:
            month = list(self._months.keys()).index(token) + 1
        elif (
            directive == '%d'
            and token_len <= 2
            and is_digit
            and 0 < int(token) <= self.calendar_converter.month_length(year, month)
        ):
            day = int(token)
        elif directive == '%Y' and token_len == 4 and is_digit:
            year = int(token)
        else:
            raise ValueError
        return self.non_gregorian_date_cls(year, month, day)

    @classmethod
    def parse(cls, datestring, settings):
        datestring = cls.to_latin(datestring)
        return super().parse(datestring, settings)



info = {
    "name": "ksh",
    "date_order": "DMY",
    "january": [
        "jan",
        "jannewa"
    ],
    "february": [
        "fäb",
        "fäbrowa"
    ],
    "march": [
        "mäz",
        "määz"
    ],
    "april": [
        "apr",
        "aprell"
    ],
    "may": [
        "mai"
    ],
    "june": [
        "jun",
        "juuni"
    ],
    "july": [
        "jul",
        "juuli"
    ],
    "august": [
        "ouj",
        "oujoß"
    ],
    "september": [
        "septämber",
        "säp"
    ],
    "october": [
        "okt",
        "oktohber"
    ],
    "november": [
        "nov",
        "novämber"
    ],
    "december": [
        "dez",
        "dezämber"
    ],
    "monday": [
        "mo",
        "mohndaach"
    ],
    "tuesday": [
        "di",
        "dinnsdaach"
    ],
    "wednesday": [
        "me",
        "metwoch"
    ],
    "thursday": [
        "du",
        "dunnersdaach"
    ],
    "friday": [
        "fr",
        "friidaach"
    ],
    "saturday": [
        "sa",
        "samsdaach"
    ],
    "sunday": [
        "su",
        "sunndaach"
    ],
    "am": [
        "uhr vörmiddaachs",
        "vm",
        "vörmeddaach"
    ],
    "pm": [
        "nm",
        "nommendaach",
        "uhr nommendaachs"
    ],
    "year": [
        "j",
        "johr"
    ],
    "month": [
        "m",
        "mohnd"
    ],
    "week": [
        "w",
        "woch"
    ],
    "day": [
        "d",
        "daach"
    ],
    "hour": [
        "s",
        "schtund",
        "std"
    ],
    "minute": [
        "m",
        "menutt",
        "min"
    ],
    "second": [
        "s",
        "sek",
        "sekond"
    ],
    "relative-type": {
        "0 day ago": [
            "hück"
        ],
        "0 hour ago": [
            "this hour"
        ],
        "0 minute ago": [
            "this minute"
        ],
        "0 month ago": [
            "diese mohnd"
        ],
        "0 second ago": [
            "now"
        ],
        "0 week ago": [
            "di woch"
        ],
        "0 year ago": [
            "diß johr"
        ],
        "1 day ago": [
            "jestere"
        ],
        "1 month ago": [
            "lätzde mohnd"
        ],
        "1 week ago": [
            "läz woch"
        ],
        "1 year ago": [
            "läz johr"
        ],
        "in 1 day": [
            "morje"
        ],
        "in 1 month": [
            "nächste mohnd"
        ],
        "in 1 week": [
            "nächste woche"
        ],
        "in 1 year": [
            "näx johr"
        ]
    },
    "relative-type-regex": {
        "\\1 year ago": [
            "vör (\\d+[.,]?\\d*) johr",
            "vör (\\d+[.,]?\\d*) johre"
        ],
        "in \\1 year": [
            "en (\\d+[.,]?\\d*) johr",
            "en (\\d+[.,]?\\d*) johre"
        ]
    },
    "locale_specific": {},
    "skip": [
        " ",
        "'",
        ",",
        "-",
        ".",
        "/",
        ";",
        "@",
        "[",
        "]",
        "|",
        "，"
    ]
}



info = {
    "name": "rn",
    "date_order": "DMY",
    "january": [
        "mut",
        "nzero"
    ],
    "february": [
        "gas",
        "ruhuhuma"
    ],
    "march": [
        "ntwarante",
        "wer"
    ],
    "april": [
        "mat",
        "ndamukiza"
    ],
    "may": [
        "gic",
        "rusama"
    ],
    "june": [
        "kam",
        "ruheshi"
    ],
    "july": [
        "mukakaro",
        "nya"
    ],
    "august": [
        "kan",
        "nyandagaro"
    ],
    "september": [
        "nyakanga",
        "nze"
    ],
    "october": [
        "gitugutu",
        "ukw"
    ],
    "november": [
        "munyonyo",
        "ugu"
    ],
    "december": [
        "kigarama",
        "uku"
    ],
    "monday": [
        "ku wa mbere",
        "mbe"
    ],
    "tuesday": [
        "kab",
        "ku wa kabiri"
    ],
    "wednesday": [
        "gtu",
        "ku wa gatatu"
    ],
    "thursday": [
        "kan",
        "ku wa kane"
    ],
    "friday": [
        "gnu",
        "ku wa gatanu"
    ],
    "saturday": [
        "gnd",
        "ku wa gatandatu"
    ],
    "sunday": [
        "cu",
        "ku w'indwi"
    ],
    "am": [
        "zmu"
    ],
    "pm": [
        "zmw"
    ],
    "year": [
        "umwaka"
    ],
    "month": [
        "ukwezi"
    ],
    "week": [
        "indwi, iyinga"
    ],
    "day": [
        "umusi"
    ],
    "hour": [
        "isaha"
    ],
    "minute": [
        "umunota"
    ],
    "second": [
        "isegonda"
    ],
    "relative-type": {
        "0 day ago": [
            "uyu musi"
        ],
        "0 hour ago": [
            "this hour"
        ],
        "0 minute ago": [
            "this minute"
        ],
        "0 month ago": [
            "this month"
        ],
        "0 second ago": [
            "now"
        ],
        "0 week ago": [
            "this week"
        ],
        "0 year ago": [
            "this year"
        ],
        "1 day ago": [
            "ejo (haheze)"
        ],
        "1 month ago": [
            "last month"
        ],
        "1 week ago": [
            "last week"
        ],
        "1 year ago": [
            "last year"
        ],
        "in 1 day": [
            "ejo (hazoza)"
        ],
        "in 1 month": [
            "next month"
        ],
        "in 1 week": [
            "next week"
        ],
        "in 1 year": [
            "next year"
        ]
    },
    "locale_specific": {},
    "skip": [
        " ",
        "'",
        ",",
        "-",
        ".",
        "/",
        ";",
        "@",
        "[",
        "]",
        "|",
        "，"
    ]
}



info = {
    "name": "af",
    "date_order": "YMD",
    "january": [
        "jan",
        "januarie"
    ],
    "february": [
        "feb",
        "februarie"
    ],
    "march": [
        "maart",
        "mrt"
    ],
    "april": [
        "apr",
        "april"
    ],
    "may": [
        "mei"
    ],
    "june": [
        "jun",
        "junie"
    ],
    "july": [
        "jul",
        "julie"
    ],
    "august": [
        "aug",
        "augustus"
    ],
    "september": [
        "sep",
        "september"
    ],
    "october": [
        "okt",
        "oktober"
    ],
    "november": [
        "nov",
        "november"
    ],
    "december": [
        "des",
        "desember"
    ],
    "monday": [
        "ma",
        "maandag"
    ],
    "tuesday": [
        "di",
        "dinsdag"
    ],
    "wednesday": [
        "wo",
        "woensdag"
    ],
    "thursday": [
        "do",
        "donderdag"
    ],
    "friday": [
        "vr",
        "vrydag"
    ],
    "saturday": [
        "sa",
        "saterdag"
    ],
    "sunday": [
        "so",
        "sondag"
    ],
    "am": [
        "vm"
    ],
    "pm": [
        "nm"
    ],
    "year": [
        "j",
        "jaar"
    ],
    "month": [
        "maand",
        "md"
    ],
    "week": [
        "week",
        "wk"
    ],
    "day": [
        "d",
        "dag"
    ],
    "hour": [
        "u",
        "uur"
    ],
    "minute": [
        "m",
        "min",
        "minuut"
    ],
    "second": [
        "s",
        "sek",
        "sekonde"
    ],
    "relative-type": {
        "0 day ago": [
            "vandag"
        ],
        "0 hour ago": [
            "hierdie uur"
        ],
        "0 minute ago": [
            "hierdie minuut"
        ],
        "0 month ago": [
            "vandeesmaand"
        ],
        "0 second ago": [
            "nou"
        ],
        "0 week ago": [
            "vandeesweek"
        ],
        "0 year ago": [
            "hierdie jaar"
        ],
        "1 day ago": [
            "gister"
        ],
        "1 month ago": [
            "verlede maand"
        ],
        "1 week ago": [
            "verlede week"
        ],
        "1 year ago": [
            "verlede jaar"
        ],
        "in 1 day": [
            "môre"
        ],
        "in 1 month": [
            "volgende maand"
        ],
        "in 1 week": [
            "volgende week"
        ],
        "in 1 year": [
            "volgende jaar"
        ]
    },
    "relative-type-regex": {
        "\\1 day ago": [
            "(\\d+[.,]?\\d*) dae gelede",
            "(\\d+[.,]?\\d*) dag gelede"
        ],
        "\\1 hour ago": [
            "(\\d+[.,]?\\d*) uur gelede"
        ],
        "\\1 minute ago": [
            "(\\d+[.,]?\\d*) min gelede",
            "(\\d+[.,]?\\d*) minute gelede",
            "(\\d+[.,]?\\d*) minuut gelede"
        ],
        "\\1 month ago": [
            "(\\d+[.,]?\\d*) maand gelede",
            "(\\d+[.,]?\\d*) maande gelede",
            "(\\d+[.,]?\\d*) md gelede"
        ],
        "\\1 second ago": [
            "(\\d+[.,]?\\d*) sek gelede",
            "(\\d+[.,]?\\d*) sekonde gelede",
            "(\\d+[.,]?\\d*) sekondes gelede"
        ],
        "\\1 week ago": [
            "(\\d+[.,]?\\d*) w gelede",
            "(\\d+[.,]?\\d*) week gelede",
            "(\\d+[.,]?\\d*) weke gelede"
        ],
        "\\1 year ago": [
            "(\\d+[.,]?\\d*) jaar gelede"
        ],
        "in \\1 day": [
            "oor (\\d+[.,]?\\d*) dae",
            "oor (\\d+[.,]?\\d*) dag",
            "oor (\\d+[.,]?\\d*) minuut"
        ],
        "in \\1 hour": [
            "oor (\\d+[.,]?\\d*) uur"
        ],
        "in \\1 minute": [
            "oor (\\d+[.,]?\\d*) min",
            "oor (\\d+[.,]?\\d*) minuut"
        ],
        "in \\1 month": [
            "oor (\\d+[.,]?\\d*) md",
            "oor (\\d+[.,]?\\d*) minuut"
        ],
        "in \\1 second": [
            "oor (\\d+[.,]?\\d*) sek",
            "oor (\\d+[.,]?\\d*) sekonde",
            "oor (\\d+[.,]?\\d*) sekondes"
        ],
        "in \\1 week": [
            "oor (\\d+[.,]?\\d*) w",
            "oor (\\d+[.,]?\\d*) week",
            "oor (\\d+[.,]?\\d*) weke"
        ],
        "in \\1 year": [
            "oor (\\d+[.,]?\\d*) jaar"
        ]
    },
    "locale_specific": {
        "af-NA": {
            "name": "af-NA"
        }
    },
    "skip": [
        " ",
        "'",
        ",",
        "-",
        ".",
        "/",
        ";",
        "@",
        "[",
        "]",
        "|",
        "，"
    ]
}
